#!/bin/bash
#################################################################
#                                                               #
# Copyright (c) 2022-2023 YottaDB LLC and/or its subsidiaries.       #
# All rights reserved.                                          #
#                                                               #
#	This source code contains the intellectual property	#
#	of its copyright holder(s), and is made available	#
#	under a license.  If you do not know the terms of	#
#	the license, please stop and do not read further.	#
#                                                               #
#################################################################
# $1 = test or shell/bash or server/server-tls/[none] [none] = server
# $2 = if $1 = test, test file (optional); otherwise, entire test suite is run
# $3 = if $1 = test, readwrite or readonly
# $2 = if $1 = shell/bash, $2 is ignored.
# $2 = if $1 = server*, it's the rest of the arguments to the %ydbwebreq (like --userpass xxx, --readwrite, log [0-3], etc.)
source /YDBGUI/dev

export USER=root # needed for maskpass
export ydb_tls_passwd_ydbgui="$(echo ydbgui | /opt/yottadb/current/plugin/gtmcrypt/maskpass | cut -d ":" -f2 | tr -d '[:space:]')"

# This is normally done by CMake, but needed as we don't have this file in the wwwroot mounted-from-host directory
if [ -f /YDBGUI/wwwroot/index.html ]; then
	if ! [ -f /YDBGUI/wwwroot/_ydbgui.manifest.json ]; then
		cp $ydb_dist/plugin/etc/ydbgui/_ydbgui.manifest.json /YDBGUI/wwwroot/_ydbgui.manifest.json
	fi
fi

if [ "$1" = "test" ]; then
	if [ ! -f /YDBGUI/wwwroot/index.html ]; then
		echo "You must mount wwwroot to /YDBGUI/wwwroot to run tests"
		exit -1
	fi
	cd /YDBGUI/wwwroot
	if [ "$3" = "readwrite" ]; then
		yottadb -run start^%ydbgui --port 8089 --tlsconfig ydbgui --log 0 --readwrite 1> /dev/null &
	else
		yottadb -run start^%ydbgui --port 8089 --tlsconfig ydbgui --log 0 1> /dev/null &
	fi
	echo "Running tests..."
	echo "Ctrl-C twice to stop..."
	if [ -n "$2" ] && [ "$2" != "--" ]; then
		exec npm test -- $2
	else
		exec npm test -- wwwroot/test/test_files/
	fi
elif [ "$1" = "shell" ] || [ "$1" = "bash" ]; then
	echo "Starting shell..."
	exec /bin/bash
else
	if [ -f /YDBGUI/wwwroot/index.html ]; then
		cd /YDBGUI/wwwroot
	else
		cd $ydb_dist/plugin/etc/ydbgui
	fi

	tls=$1
	shift 1

	echo "Starting the Server..."
	echo "Ctrl-\ to stop"
	if [ "$tls" = "server-tls" ] || [ "$tls" = "tls-server" ]; then
		exec yottadb -run start^%ydbgui --port 8089 --tlsconfig ydbgui $@
	else
		exec yottadb -run start^%ydbgui --port 8089 $@
	fi
fi
