<!--
/****************************************************************
 *                                                              *
 * Copyright (c) 2022-2023 YottaDB LLC and/or its subsidiaries. *
 * All rights reserved.                                         *
 *                                                              *
 * This source code contains the intellectual property          *
 * of its copyright holder(s), and is made available            *
 * under a license.  If you do not know the terms of            *
 * the license, please stop and do not read further.            *
 *                                                              *
 ****************************************************************/
-->

# REST Endpoints 

## Overview

| Method | Endpoint                                                                  | Params           | Description                                                                      |
|--------|---------------------------------------------------------------------------|------------------|----------------------------------------------------------------------------------|
| GET    | [api/dashbboard/getAll](#get-apidashbboardgetall)                         |                  | Retrieves full info on all regions, devices and system                           |
| GET    | [api/regions/{region}](#get-apiregionsregion)                             |                  | Retieves info about a specific region                                            
| DELETE | [api/regions/{region}](#get-apiregionsregion)                             | deleteFiles=true | Deletes a specific region                                                        
| POST   | [api/regions/{region}/createDb](#post-apiregionsregioncreatedb)           |                  | Creates a database in the specified region                                       
| POST   | [api/regions/{region}/extend](#post-apiregionsregionextend)               | blocks=_nnn_     | Extends the specified region for the given amount of blocks                      
| POST   | [api/regions/{region}/journalSwitch](#post-apiregionsregionjournalswitch) | mode=_on_  _off_ | Switches the journal `on` or `off` in the given region                           
| GET    | [api/dashboard/getTemplates](#get-apidashbboardgettemplates)              |                  | Returns the GDE templates and their numeric limits                               
| POST   | [api/regions/parseNamespace](#post-apiregionsparsenamespace)              |                  | Verify a namespace                                                               
| POST   | [api/regions/validatePath](#post-apiregionsvalidatepath)                  |                  | Validate a filepath                                                              
| POST   | [api/regions/add](#post-apiregionsadd)                                    |                  | Creates a region                                                                 
| POST   | [api/regions/{region}/edit](#post-apiregionsregionedit)                   |                  | Edits a region                                                                   
| GET    | [api/test/error](#get-apitesterror)                                       |                  | Triggers an M error and returns the error description in the status 500 response 
| POST   | [api/test/error](#post-apitesterror)                                      |                  | Triggers an M error and returns the error description in the status 500 response 
| DELETE | [api/test/error](#delete-apitesterror)                                    |                  | Triggers an M error and returns the error description in the status 500 response 
| GET    | [api/regions/locks/getAll](#get-apiregionslocksgetall)                    |                  | Returns all the locks and waiters across the all regions                         
| POST   | [api/regions/locks/clear](#post-apiregionslocksclear)                     | namespace=       | Clear the lock specified in the "namespace" parameter                            
| POST   | [api/os/processes/{pid}/terminate](#post-apiosprocessespidterminate)      | {pid}            | Terminate the process specified in the "pid" parameter using the SIGINT signal   
| GET    | [api/globals/dollarData](#get-apiglobalsdollardata)                       | globalName=      | Returns the $data value of a global                                              
| GET    | [api/globals/find](#get-apiglobalsfind)                                   | mask= &size=     | Query the globals against a mask                                                 
| POST   | [api/globals/traverse](#post-apiglobalstraverse)                          |                  | Traverse a global with paramteres passed in the body section                     
| GET    | [api/routines/find](#get-apiroutinesfind)                                 | mask=            | Query the routines against a mask                                                
| GET    | [api/routines/{routine}](#get-routinesroutine)                            |                  | Gets a routine                                                                   
| GET    | [api/server/mode](#get-servermode)                                        |                  | Returns the current server mode (RO or RW)                                       
| POST   | [api/regions/maintenance/defrag](#post-regionsmaintenancedefrag)          |                  | Performs a DEFRAG operation on one or more regions                               
| POST   | [api/regions/maintenance/integ](#post-regionsmaintenanceinteg)            |                  | Performs an INTEG operation on one or more regions                               

### `GET` api/dashboard/getAll

Retrieves information about all regions, all used disk devices, system information and soft3are version

Parameters:
`none`

Returns:

SUCCESS
````js
res = {
    result: 'OK',
    data: {         // The dashboard data 
        devices: [],
        gld: {},
        regions: {
            regionName: {
                dbAccess: {
                    data: []
                },
                dbFile: {
                    data: [],
                    flags: {
                        
                    },
                    usage: {}
                },
                journal: {
                    data:[],
                    flags: {}
                },
                locks: {},
                replication: {},
                stats: {
                    csa: {},
                    journal: {},
                    locks: {},
                    logicalOperations: {},
                    transactions: {}
                }
            }
        },
        systemInfo: {},
        ydb_version: ''
    } 
}
````

ERROR

````js
res = {
    result: 'ERROR',
    error : {description: ''} // The error description
}
````

WITH WARNINGS

````js
res = {
    result: 'WARNING',
    data : {
        warnings: [
            // The warning(s) description(s)
        ],
        // ... same as SUCCESS
    } 
}
````

---


### `GET` api/regions/{region}

Retrieves information about a specific region

Parameters: 

`none`

Returns:

SUCCESS
````js
res = {
    result: 'OK',
    data: {          
        dbAccess: {
            data: []
        },
        dbFile: {
            data: [],
            flags: {
                
            },
            usage: {}
        },
        journal: {
            data:[],
            flags: {}
        },
        locks: {},
        replication: {},
        stats: {
            csa: {},
            journal: {},
            locks: {},
            logicalOperations: {},
            transactions: {}
        }
    }
}
````

ERROR

````js
res = {
    result: 'ERR',
    error : {description: ''} // The error description
}
````

WITH WARNINGS

````js
res = {
    result: 'WARNING',
    data : {
        warnings: [
            // The warning(s) description(s)
        ],
        // ... same as SUCCESS
    } 
}
````
---

### `DELETE` api/regions/{region}

Deletes the specified region

Parameters: 

`deleteFiles=true`

If deleteFiles=true also journal and database files are deleted.

Body: 

`none`

---

### `POST` api/regions/{region}/createDb

Creates the database file specified in the given region

Parameters: 

`none`

Body: 

`none`

Returns:

SUCCESS

````js
res = {
    result: 'OK'
}
````

ERROR

````js
res = {
    result: 'ERR',
    error : {
        description: '',    // The error description
        dump : []           // Optional response from the shell
     }
}
````

---

### `POST` api/regions/{region}/extend

Extends the database file in the specified region by the passed amount of blocks

Parameters: 

`number` blocks=_nnn_

Body: 

`none`

Returns:

SUCCESS

````js
res = {
    result: 'OK'
}
````

ERROR

````js
res = {
    result: 'ERR',
    error : {description: ''} // The error description
}
````

---

### `POST` api/regions/{region}/journalSwitch

Switches the journaling on or off in the specified region


Parameters: 

`string` turn=on|off

Body: 

`none`

Returns:

SUCCESS

````js
res = {
    result: 'OK'
}
````

ERROR

````js
res = {
    result: 'ERR',
    error : {description: ''} // The error description
}
````

---

### `GET` api/dashbboard/getTemplates

Retrieves all the GDE templates and their associated min/max values (when applicable)

Parameters: 

`none`

Returns:

SUCCESS

````js
res = {
    result: 'OK',
    data: {
        region: {           // each entry is an object:
                            // { min: 0, max: 0, value: 0} 
        },
        segment: {
            BG: {
                
            },
            MM: {
                
            }
        }
    }
}
````

ERROR

````js
res = {
    result: 'ERR',
    error : {description: ''} // The error description
}
````

---

### `POST` api/regions/parseNamespace

Validates a namespace in the GDE.

Parameters: 

`none`

Body: 
````js
res = {
    namespace: ''
}
````

Returns:

SUCCESS

````js
res = {
    result: 'OK',
    data: {
        parseResult: ''             // literal 'OK' or the error string returned by the GDE
    }
}
````

ERROR

````js
res = {
    result: 'ERR',
    error : {description: ''} // The error description
}
````


---

### `POST` api/regions/validatePath

Validates a path and optionally expand the directories if Env Vars are used.
It will additionally check for filename existence.

Parameters: 

`none`

Body: 
````js
res = {
    path: ''
}
````
Returns:

SUCCESS

````js
res = {
    result: 'OK',
    data: {
        validation: '',             // an (optionally expanded) path if path is valid, 
                                    // empty string if invalid
        fileExist: ''               // the full (optionally expanded) path if the file already exists
                                    // empty string if file does NOT exists
    }
}
````

ERROR

````js
res = {
    result: 'ERR',
    error : {description: ''} // The error description
}
````


---

### `POST` api/regions/{region}/edit

Edits a region to the system. 

Parameters: 

`none`

Body: 
````js
body = {
    changeAccessMethod: false,              
    changeJournal: true,                     
    journalFilename: '',
    journalUpdateGde: true,
    journalUpdateMupip: false,
    dbAccess: {
        region: [],
        journal: []
    },
    newAccessMethod: '',
    names: [],
    newJournalEnabled: true,
    regionName: '',
    segmentData: [],
    segmentFilename: '',
    updateGde: false
}
````
Returns:

SUCCESS

````js
res = {
    result: 'OK'
}
````

ERROR

````js
res = {
    result: 'ERR',
    error : {
        description: '',    // The error description
        dump: []            // Array of string with the dump of the shell response
    } 
}
````

WITH WARNINGS

````js
res = {
    result: 'WARNING',
    data : {
        warnings: [
            {
                description: '',        // The warning(s) description(s)
                dump: []                // Array of string with the dump of the shell response
            }
            
        ]
    } 
}
````


---

### `POST` api/regions/add

Adds a new region to the system. 

Parameters: 

`none`

Body: 
````js
body = {
    regionName: '',
    segmentTypeBg: true,
    journalEnabled: true,
    segmentData: [],
    dbAccess: {
        region: [],
        journal: []
    },
    templates: {
        updateTemplateDb: true,
        updateTemplateJournal: false
    },
    names: [],
    segmentFilename: '',
    journalFilename: '',
    postProcessing: {
        createDbFile: true,
        switchJournalOn: true
    }
}
````
Returns:

SUCCESS

````js
res = {
    result: 'OK'
}
````

ERROR

````js
res = {
    result: 'ERR',
    error : {description: ''} // The error description
}
````

WITH WARNINGS

````js
res = {
    result: 'WARNING',
    data : {
        warnings: [
            {
                description: '',        // The warning(s) description(s)
                dump: []                // array with shell responses for debugging
            }
            
        ]
    } 
}
````


---

### `GET` api/test/error

Triggers an M error and returns the error description in the status 500 response

Parameters: 

`none`

Returns:

ERROR

````js
res = {
    apiVersion: "1.0",
    error: {
        code: 500,
        errors: [
            {
                logID: 99999,
                mcode: " s a=1/0",
                message: "150373210,error+1^%ydbguiRest,%YDB-E-DIVZERO, Attempt to divide by zero",
                place: "error+1^%ydbguiRest",
                reason: ",M9,Z150373210,"
            }
        ],
        message: "Internal Server Error",
        request: "DELETE /api/test/error/ "
    }
}
````

---

### `POST` api/test/error

Triggers an M error and returns the error description in the status 500 response

Parameters: 

`none`

Body: 

`none`

Returns:

ERROR

````js
res = {
    apiVersion: "1.0",
    error: {
        code: 500,
        errors: [
            {
                logID: 99999,
                mcode: " s a=1/0",
                message: "150373210,error+1^%ydbguiRest,%YDB-E-DIVZERO, Attempt to divide by zero",
                place: "error+1^%ydbguiRest",
                reason: ",M9,Z150373210,"
            }
        ],
        message: "Internal Server Error",
        request: "DELETE /api/test/error/ "
    }
}
````

---

### `DELETE` api/test/error

Triggers an M error and returns the error description in the status 500 response

Parameters: 

`none`

Body: 

`none`

Returns:

ERROR

````js
res = {
    apiVersion: "1.0",
    error: {
        code: 500,
        errors: [
            {
                logID: 99999,
                mcode: " s a=1/0",
                message: "150373210,error+1^%ydbguiRest,%YDB-E-DIVZERO, Attempt to divide by zero",
                place: "error+1^%ydbguiRest",
                reason: ",M9,Z150373210,"
            }
        ],
        message: "Internal Server Error",
        request: "DELETE /api/test/error/ "
    }
}
````

---

### `GET` api/regions/locks/getAll

Returns all the locks, waiters and related processes information

Parameters: 

`none`

Returns:

SUCCESS

````js
res = {
    result: 'OK',
    locks: [
        {
            namespace: 'TEST1',
            region: 'YDBOCTO',
            pid: 1,
            waiters: [
                11,
                12,
                13,
            ]
        },
        {
            namespace: 'TEST2',
            region: 'DEFAULT',
            pid: 2,
            waiters: [
                21,
                22
            ]
        },
        {
            namespace: 'TEST3',
            region: 'YDBAIM',
            pid: 3,
            waiters: [
                31,
                32,
                33
            ]
        },
    ],
    pids: [
        {
            pid: 1,
            userId: 'pid1',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
        {
            pid: 2,
            userId: 'pid2',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
        {
            pid: 3,
            userId: 'pid3',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
        {
            pid: 11,
            userId: 'pid11',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
        {
            pid: 12,
            userId: 'pid12',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
        {
            pid: 13,
            userId: 'pid13',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
        {
            pid: 21,
            userId: 'pid21',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
        {
            pid: 22,
            userId: 'pid22',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
        {
            pid: 23,
            userId: 'pid23',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
        {
            pid: 31,
            userId: 'pid31',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
        {
            pid: 32,
            userId: 'pid32',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
        {
            pid: 33,
            userId: 'pid33',
            processName: 'yottadb',
            PPID: 0,
            time: '00:00:00'
        },
    ],
    regions: [
        {
            name: 'DEFAULT',
            estimatedFreeLockSpace: '100% of 220 pages',
            processesOnQueue: '0/880',
            slotsInUse: '0/597',
            slotsBytesInUse: '0/28080'
        },
        {
            name: 'YDBOCTO',
            estimatedFreeLockSpace: '100% of 220 pages',
            processesOnQueue: '0/880',
            slotsInUse: '0/597',
            slotsBytesInUse: '0/28080'
        },
        {
            name: 'YDBAIM',
            estimatedFreeLockSpace: '100% of 220 pages',
            processesOnQueue: '0/880',
            slotsInUse: '0/597',
            slotsBytesInUse: '0/28080'
        }
    ]
}
````

ERROR

````js
res = {
    result: 'ERR',
    error : {description: '',   // The error description
    dump: []                    // Eventual dump of shell response
} 
}
````

---

### `POST` api/regions/locks/clear

Clears the lock identified in the namepsace parameter.

Parameters: 

`namespace=`

Specify the namespace to clear.

Body: 

`none`

Returns:

SUCCESS

````js
res = {
    result: 'OK'
}
````

ERROR 

````js
res = {
    result: 'ERROR',
    error : {description: ''} // The error description
}
````


---

### `POST` api/os/processes/{pid}/terminate

Terminates the process identified by the `pid` parameter. 

Note: the process must be a yottadb process or it won't be terminated and an error will be returned.

Parameters: 

`none`

Body: 

`none`

Returns:

SUCCESS

````js
res = {
    result: 'OK',
}
````


ERROR

````js
res = {
    result: 'ERROR',
    error : {description: ''} // The error description
}
````

---

### `GET` api/globals/find

Searches globals in the global directory.

Parameters: 

`mask`
The search string WITHOUT the ^ char (no * or ? allowed, just the search string). It can be empty (to start from the beginning).

`size`
The number of records returned by the call. If omitted or 0, it will return all the records.

Body: 

`none`

Returns:

SUCCESS

````js
res = {
    result: 'OK',
    data: []            //An array of strings with the global names
}
````


ERROR

````js
res = {
    result: 'ERROR',
    error : {description: ''} // The error description
}
````

---

### `POST` api/globals/traverse

Traverse a global and returns the entries and values.

It is possible to use the * and the comma (,) to format / restrict the data returned.\

It is possible to use ranges, with or without start/end params

Each row in the returned list includes:
- The global name
- An array of subscripts
- The(eventual) value of the node
- The $data of the node (always > 0)

Parameters: 

`none`

Body: 

````js
body = {
    namespace: {
        global: '^MYGLOBAL',            // The global name prefixed by the ^ char
        subscripts: [
            {
                type: '',               // The type of subscript: (always present)
                                        // V for value: node value must be set
                                        // , (comma) for placeholder : no other node needs to be set
                                        // * (star) for all children (MUST be the last subscript : no other node needs to be set
                                        // :range both value and valueMax can be empty
                                        
                value: '',              // The value. It can be empty in a range, comma or star types
                                        // Strings are ALWAYS quoted
                                        
                valueEnd: ''            // The end value of a range. Can be empty if value is not empty
                                        // Strings are ALWAYS quoted 
            }
        ]
    },
    size: 0,                            // The number of records to return. If missing, it will default to 100
    offset: '',                         // If going DOWN, it is the string path of the last record
                                        // IMPORTANT: if you are re-submitting the call, you MUST RESEND ALSO the same namespace you used in the previous call or failure may occur 
    upOffset: 0,                        // If going UP, it is the numeric record to start with
                                        // IMPORTANT: if you are re-submitting the call, you MUST RESEND ALSO the same namespace you used in the previous call or failure may occur 
    direction: '',                      // 'up' OR 'down' OR 'end'. If omitted or empty defaults to 'down'
    dataSize: 512,                     // The maximum size of the returned data. If omitted, default is 512 chars
}
````

Returns:

SUCCESS

````js
res = {
    result: 'OK',
    data: []            //An array of strings with the global names
}
````


ERROR

````js
res = {
    result: 'ERROR',
    error : {description: ''} // The error description
}
````

---

### `GET` api/globals/dollarData

Returns the $data value of a global.
If name is not valid, it returns -1.

Parameters: 

`globalName`
The name of the global, including the ^ character.

Body: 

`none`

Returns:

SUCCESS

````js
res = {
    result: 'OK',
    data: value            //The $data value OR -1 if syntax error
}
````
---

### `GET` api/globals/getData

Returns the entire node value of the passed parameter PATH

Parameters: 

`path`
The path you wish to retrieve the value from.

Body: 

`none`

Returns:

SUCCESS

````js
res = {
    result: 'OK',
    data: value            //The value
}
````
---

### `GET` api/routines/find

Returns a list of routines found, using the provided mask to filter it

Parameters: 

`mask`
The mask to filter the list. It must have at least 2 chars (excluding wildcards). % (which is _y) may be one of the character.
> Note: the char _ must be used instead of the %

Wildcards are: * and _ ?

Body: 
  
`none`

Returns:

SUCCESS

No routines found:
````js
res = {
    result: 'OK',
    data: 0
}
````

Routines found:
````js
res = {
    result: 'OK',
   data: {
        '%ydb': {
            'object': {
                '%ydb': '/YDBGUI/objects/'
            },
            'path': '/YDBGUI/routines/',
            'size': 8
        },
        "%ydbQuantumQueries": {
            "path": "/data/r1.35_x86_64/r/",
            "size": 11
        }
   }
}
````

---

### `GET` api/routines/routine

Returns a routine
> Note: the % character must be replaced with _ (underscore)

Parameters: 


Body: 

`none`

Returns:

SUCCESS

````js
res = {
          "data": {
              "file": [
                  "%ydbgui ; Start-up code",
                  ""
              ],
              "object": {
                  "props": {
                      "accessRightsHRF": "-rw-r--r--",
                      "accessRightsOctal": 644,
                      "birth": "-",
                      "filename": "_ydbgui.o",
                      "lastAccess": "2022-10-29 18:58:36.541483713 +0000",
                      "lastModified": "2022-10-29 18:58:36.541483713 +0000",
                      "lastStatusChange": "2022-10-29 18:58:36.541483713 +0000",
                      "ownerGroupName": "root",
                      "ownerUserName": "root",
                      "path": "/YDBGUI/objects/",
                      "size": 12544
                  }
              },
              "props": {
                  "accessRightsHRF": "-rwxrwxrwx",
                  "accessRightsOctal": 777,
                  "birth": "-",
                  "filename": "_ydbgui.m",
                  "lastAccess": "2022-10-25 09:48:23.211996500 +0000",
                  "lastModified": "2022-10-25 09:16:41.130358300 +0000",
                  "lastStatusChange": "2022-10-25 09:16:41.130358300 +0000",
                  "ownerGroupName": "root",
                  "ownerUserName": "root",
                  "path": "/YDBGUI/routines/",
                  "size": 5039
              }
          },
    "result": "OK"
}
````

---

### `GET` api/server/mode

Returns the current server mode (RO or RW)

Parameters:

`none`

Body:

`none`

Returns:

SUCCESS

````js
res = {
    serverMode: 'RO'
}
````

---

### `POST` api/regions/maintenance/defrag

Performs a DEFRAG operation on one or more regions.

Parameters:

`none`

Body:

````js
body = {
    fillFactor: 0,                      // the fillFactor parameter, between 0 and 100
    indexFillFactor: 0,                 // the indexFillFactor parameter, between 0 and 100
    regions: [                          // the region list, an array of strings
        'default'
    ],
    recover: false,                     // the `recover` parameter, boolean, default is false
    resume: false                       // the `resume` parameter, boolean, default is false
}
````

Returns:

SUCCESS

````js
res = {
    command: 'string',                  // the command that has been executed in the server
    result: 'OK',
    data: {                             // one key per region
        DEFAULT: {
            byGlobal: {                 // one key per global
                a: {
                    coalesced: 0,
                    extended: 0,
                    freed: 0,
                    processed: 0,
                    reused: 0,
                    split: 0,
                    swapped: 0
                }
            },
            incompleteGlobalList: false, // used in resume, to indicate that the global list is incomplete due to earlier processing
            totals: {
                coalescend: 0,
                extended: 0,
                freed: 0,
                processed: 0,
                reused: 0,
                split: 0,
                swapped: 0
            }
        }
    },
    payloadRegions: []                  // internally used by the async executor
}
````

ERROR

````js
res = {
    result: 'ERROR',
    error: {description: ''}           // The error description
}
````

---

### `POST` api/regions/maintenance/integ

Performs an INTEG operation on one or more regions.

Parameters:

`none`

Body:

````js
body = {
    regions: [                          // the region list, an array of strings
        'default'
    ],
    type: 'fast',                       // the `type` parameter, which can be `fast` or `complete`. Default is `complete`.
    reporting: 'full',                  // the `reporting` parameter, which can be `summary` or `full`. Default is `summary`.
    keyRange: true,                     // the `keyRange` parameter, boolean. Default is `true`.
    stat: false,                        // the `stat` parameter, boolean. Default is `false`.
    mapErrors: 10,                      // the `mapErrors` limit. Value is from 0 (unlimited) to 1,000,000. Default is 10.
    keySizeErrors: 10,                  // the `keySizeErrors` limit. Value is from 0 (unlimited) to 1,000,000. Default is 10.
    transactionErrors: 10,              // the `transactionErrors` limit. Value is from 0 (unlimited) to 1,000,000. Default is 10.
}
````

Returns:

SUCCESS

````js
res = {
    command: 'string',                   // the command that has been executed in the server
    result: 'OK',
    data: []                            // an array of strings with the result dump
}
````

ERROR

````js
res = {
    result: 'ERROR',
    error: {description: ''}           // The error description
}
````
