%ydbguiGlobals ; Globals core; 05-07-2021
	;#################################################################
	;#                                                               #
	;# Copyright (c) 2022-2023 YottaDB LLC and/or its subsidiaries.       #
	;# All rights reserved.                                          #
	;#                                                               #
	;#   This source code contains the intellectual property         #
	;#   of its copyright holder(s), and is made available           #
	;#   under a license.  If you do not know the terms of           #
	;#   the license, please stop and do not read further.           #
	;#                                                               #
	;#################################################################
	;
	quit
	;
; ****************************************************************
; find(mask)
;
; PARAMS:
; mask (string) mask without ^
; size (number) number of records to return, 0 will return all
;
; RETURNS
; JDOM res array with globals list
; ****************************************************************
find(mask,size)
	new res,gname,cnt,gnameStart,gnameStop
	;
	set cnt=0
	set gname="^"_mask
	if $zlength(mask)<2 do
	. set gnameStart="^%"
	. set gnameEnd="^zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz"
	. ;
	. if $data(^%) set res("data",$increment(cnt))="%"
	. ;
	else  do
	. set gnameStart=$zextract(gname,1,$zlength(gname)-1)
	. set gnameEnd=$zextract(gname,1,$zlength(gname)-1)_"zzzzzzzzzz"
	;
	for  set gnameStart=$order(@gnameStart) quit:gnameStart=""!(gnameStart]gnameEnd)  do  quit:(size>0)&(cnt>=size)
	. if $zlength(mask)<2 set res("data",$increment(cnt))=gnameStart
	. else  set:$zextract(gnameStart,2,$zlength(mask)+1)=mask res("data",$increment(cnt))=$zextract(gnameStart,2,99)
	;
	set res("result")="OK"
	;
	quit *res
	;
	;
; ****************************************************************
; traverse(params)
;
; PARAMS:
; params			byRef
;
; RETURNS
; JDOM res array with globals list
; ****************************************************************
traverse(params)
	new subs,cnt,hasStar,returnAllNodes,pathSubs,recordCount,maxRecords,quitNow,res,subsCount
	new bashPath,subStr,found,subNoQuotes,offsetSubs,offsetPath,continue,gname,direction,skipCounter,upOffset
	new bufferSize
	;
	; init variables
	set (basePath,gname)=params("namespace","global")
	set offsetPath=$get(params("offset"))
	set subStr=""
	set maxRecords=params("size")
	set (recordCount,quitNow,continue,skipCounter,bufferSize)=0
	set returnAllNodes=$select($get(params("returnAll"))="true":1,1:0)
	merge subs=params("namespace","subscripts")
	set subsCount=$order(subs(""),-1)
	set hasStar=$select($get(subs($order(subs(""),-1),"type"))="*":1,1:0)
	set direction=params("direction")
	set upOffset=+$get(params("upOffset"))
	;
	; detect comma or range
	set (found,level)=0
	set cnt="" for  set cnt=$order(subs(cnt)) quit:cnt=""!(found)  do
	. if subs(cnt,"type")=","!(subs(cnt,"type")=":") set found=1
	;
	; Creates the base path
	if $zlength(subStr) set basePath=basePath_"("_$zextract(subStr,1,$zlength(subStr)-1)_")"
	;
	; Check the root node
	if (returnAllNodes=0&($data(@basePath)=1!($data(@basePath)=11)))!(returnAllNodes=1) do
	. set skipCounter=skipCounter+1
	. if ((offsetPath=""!(continue)!((direction=-1))&(skipCounter>upOffset))&(level>=subsCount))!((subsCount=1)&(subs(1,"type")="*")) do
	. . ; log the record
	. . do logRecord(basePath)
	;
	; start walking
	do traverseWalker(basePath)
	;
	; adjust record count for continue mode
	if offsetPath'="" do
	. set recordCount=recordCount-1
	. kill res("data","list",$order(res("data","list","")))
	;
	; update counter
	set res("data","recordCount")=recordCount
	;
	quit *res
	;
traverseWalker(path)
	new ix,quitAll
	;
	set quitAll=0
	set level=level+1
	;
	; if fixed value, append to path and continue traversing
	if $get(subs(level,"value"))'="",$get(subs(level,"type"))="V" do  goto traverseWalkerQuit
	. set path=$name(@path@(subs(level,"value")))
	. if (offsetPath'=""&(offsetPath=path)) set continue=1			; do we need to continue
	. do traverseWalker(path)
	. ;
	. if (returnAllNodes=0&($data(@path)=1!($data(@path)=11)))!(returnAllNodes=1) do  quit:quitNow
	. . set skipCounter=skipCounter+1
	. . if ((offsetPath=""!(continue)!(direction=-1)&(skipCounter>upOffset))!(direction=0))&(level>=subsCount) do
	. . . ; log the record
	. . . do logRecord(path)
	. . . ; Check record limits and quit if done
	. . . if direction=0 quit
	. . . if recordCount=$select(continue:maxRecords+1,1:maxRecords) set quitNow=1
	;
	if $get(subs(level,"value"))'="" set ix=subs(level,"value")
	else  set ix=""
	;
	for  set ix=$order(@path@(ix)) quit:ix=""!(quitAll)!(quitNow)  do
	. if $get(subs(level,"type"))="V",$get(subs(level,"value"))'="",(ix>subs(level,"value")!(ix]subs(level,"value"))) quit
	. if $get(subs(level,"type"))=":",$get(subs(level,"valueEnd"))'="",(ix>subs(level,"valueEnd")!(ix]subs(level,"valueEnd"))) quit
	. if $data(subs(level))=0,hasStar=0 set quitAll=1 quit
	. ;
	. set newPath=$name(@path@(ix))
	. if (offsetPath'=""&(offsetPath=newPath)) set continue=1
	. if (returnAllNodes=0&($data(@newPath)=1!($data(@newPath)=11)))!(returnAllNodes=1) do  quit:quitNow
	. . set skipCounter=skipCounter+1
	. . if ((offsetPath=""!(continue)!(direction=-1)&(skipCounter>upOffset))!(direction=0))&(level>=subsCount) do
	. . . ; log the record
	. . . do logRecord(newPath)
	. . ;
	. . ; Check record limits and quit if done
	. . if direction=0 quit
	. . if recordCount=$select(continue:maxRecords+1,1:maxRecords) set quitNow=1
	. ;
	. ; continue walking is * or $data>9 or ,
	. if ($get(subs(level,"type"))=","!(hasStar)),$data(@newPath)>9 do traverseWalker(newPath)
	;
traverseWalkerQuit
	set level=level-1
	;
	quit
	;
	;
logRecord(path)
	; log the record
	set recordCount=recordCount+1
	set res("data","list",recordCount,"path")=path
	set res("data","list",recordCount,"$data")=$data(@path)
	set res("data","list",recordCount,"global")=gname
	; append the subscripts
	do extractSubs(path,$name(res("data","list",recordCount,"subscripts")))
	; and the $data and value
	if $data(@path)=1!($data(@path)=11) do
	. if $length($get(@path))>params("dataSize") set res("data","list",recordCount,"value")=$extract($get(@path),1,params("dataSize")),res("data","list",recordCount,"hasMore")="true"
	. else  set res("data","list",recordCount,"value")=$get(@path),res("data","list",recordCount,"hasMore")="false"
	else  set res("data","list",recordCount,"value")="null",res("data","list",recordCount,"hasMore")="false"
	;
	if direction=0 do
	. if bufferSize<maxRecords set bufferSize=bufferSize+1
	. else  kill res("data","list",recordCount-bufferSize)
	;
	quit
	;
	;
extractSubs(path,subs)
	; Creates the subs node by parsing the path
	quit:$qlength(path)=0
	;
	new ix
	;
	for ix=1:1:$qlength(path) set @subs@(ix)=$qsubscript(path,ix)
	;
	quit
	;
	;
; ****************************************************************
; globalsData(globalName)
;
; PARAMS:
; globalName (string) global name, including the ^
;
; RETURNS
; {data: $data} OR {data: -1) if error
; ****************************************************************
globalData(globalName)
	new res
	;
	set res("data")=-1
	set globalName=$ztranslate(globalName,"_","%")
	new $etrap
	set $etrap="goto globalDataQuit"
	;
	set res("data")=$data(@globalName)
	;
globalDataQuit
	set $ecode=""
	set res("result")="OK"
	;
	quit *res
