/****************************************************************
 *                                                              *
 * Copyright (c) 2022-2023 YottaDB LLC and/or its subsidiaries.      *
 * All rights reserved.                                         *
 *                                                              *
 * This source code contains the intellectual property          *
 * of its copyright holder(s), and is made available            *
 * under a license.  If you do not know the terms of            *
 * the license, please stop and do not read further.            *
 *                                                              *
 ****************************************************************/

// ********************************
// _getZroutines
// ********************************
app.REST._getZroutines = () => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('get', 'server/zroutines', {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        }, true)
    })
};

// ********************************
// journal file switch
// ********************************
app.REST._JournalFilesSwitch = region => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'regions/' + region + '/journalFileSwitch', {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// _getJournalInfo
// ********************************
app.REST._getJournalInfo = (region) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('get', 'regions/' + region + '/getJournalInfo', {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        }, true)
    })
};

// ********************************
// _backup
// ********************************
app.REST._backup = (payload) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'regions/maintenance/backup', payload, data => {
            resolve(data)

        }, err => {
            reject(err)
        }, true)
    })
};

// ********************************
// _pathExpand
// ********************************
app.REST._pathExpand = (path, dirsOnly = true) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'os/fs/pathExpand', {path: path, dirsOnly: dirsOnly}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// _ls
// ********************************
app.REST._ls = (path, type = 'D', mask = '*') => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'os/fs/ls', {
            path: path,
            type: type,
            mask: mask
        }, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// _integ
// ********************************
app.REST._integ = (params) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'regions/maintenance/integ', params, data => {
            resolve(data)

        }, err => {
            reject(err)
        }, true)
    })
};

// ********************************
// _defrag
// ********************************
app.REST._defrag = (params) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'regions/maintenance/defrag', params, data => {
            resolve(data)

        }, err => {
            reject(err)
        }, true)
    })
};

// ********************************
// _getRoutine
// ********************************
app.REST._getRoutine = (routine) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'routines/', {
            routine: routine.replace(/%/g, '_'),
            zroutines: app.ui.rViewer.searchPath.zroutines
        }, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// _findRoutines
// ********************************
app.REST._findRoutines = (mask) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'routines/find', {
            mask: mask.replace(/%/g, '_'),
            zroutines: app.ui.rViewer.searchPath.zroutines
        }, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// _getData
// ********************************
app.REST._getData = (path) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('get', 'globals/getData?path=' + path, {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// _dollarData
// ********************************
app.REST._dollarData = (globalName) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('get', 'globals/dollarData?globalName=' + globalName, {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// _globalsFind
// ********************************
app.REST._globalsFind = (globalMask, size = 0) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('get', 'globals/find?mask=' + globalMask + '&size=' + size, {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// _globalsTraverse
// ********************************
app.REST._globalsTraverse = (payload) => {
    console.log(payload)
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'globals/traverse', payload, data => {
            resolve(data)

        }, err => {
            reject(err)
        }, true)
    })
};

// ********************************
// terminates a process
// ********************************
app.REST._terminateProcess = (PID) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'os/processes/' + PID + '/terminate', {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// clears a lock
// ********************************
app.REST._clearLock = (lockNamespace) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'regions/locks/clear?namespace=' + lockNamespace, {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// Get locks info
// ********************************
app.REST._getAllLocks = () => {
    return new Promise(function (resolve, reject) {

        app.REST.execute('get', 'regions/locks/getAll', {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// Add region
// ********************************
app.REST._addRegion = (payload) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'regions/add', payload, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// Edit region
// ********************************
app.REST._editRegion = (payload) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'regions/' + payload.regionName + '/edit', payload, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// Validate path
// ********************************
app.REST._validatePath = (path) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'regions/validatePath', {path: path}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// Parse namespace
// ********************************
app.REST._parseNamespace = (namespace) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'regions/parseNamespace', {namespace: namespace}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// Get templates
// ********************************
app.REST._getTemplates = () => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('get', 'dashboard/getTemplates', {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// Get dashboard info
// ********************************
app.REST._dashboardGetAll = () => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('get', 'dashboard/getAll', {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// Get region info
// ********************************
app.REST._regionGet = (region) => {
    return new Promise(function (resolve, reject) {

        app.REST.execute('get', 'regions/' + region, {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// delete region
// ********************************
app.REST._regionDelete = (region, withDelete) => {
    return new Promise(function (resolve, reject) {

        app.REST.execute('delete', 'regions/' + region + '?' + (withDelete === true ? 'deleteFiles=true' : ''), {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// create region Db file
// ********************************
app.REST._regionCreateDbFile = (region) => {
    return new Promise(function (resolve, reject) {

        app.REST.execute('post', 'regions/' + region + '/createDb', {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// extend region(blocks)
// ********************************
app.REST._regionExtend = (region, blocks) => {
    return new Promise(function (resolve, reject) {
        app.REST.execute('post', 'regions/' + region + '/extend?blocks=' + blocks, {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// switch region(mode)
// state: 'on' || 'off'
// ********************************
app.REST._JournalSwitch = (region, mode) => {
    return new Promise(function (resolve, reject) {
        if (mode !== 'on' && mode !== 'off') {
            reject({
                err: {
                    description: 'The mode parameter is invalid: ' + mode
                }
            });

            return
        }

        app.REST.execute('post', 'regions/' + region + '/journalSwitch?turn=' + mode, {}, data => {
            resolve(data)

        }, err => {
            reject(err)
        })
    })
};

// ********************************
// Global Ajax handlers
// ********************************
app.REST.execute = (type, command, DATA = {}, okCallback, errCallback, longTimeout = false) => {
    //debug info
    console.log('REST command: ' + command);

    $.ajax({
        url: '/api/' + command,
        contentType: 'application/json',
        dataType: 'json',
        data: JSON.stringify(DATA),
        type: type,
        crossDomain: true,
        timeout: longTimeout === true ? app.userSettings.rest.timeout.long : app.userSettings.rest.timeout.short,
        success: (data) => {
            //debug info
            console.log('REST success: ');
            console.log(data)
            okCallback(data);
        },
        error: (err) => {
            //debug info
            console.log('REST error: ');
            console.dir(err);
            errCallback(err);
        }
    });
};

app.REST.parseError = err => {
    let message = 'The following error occurred while fetching the data: ';

    if (err.status !== undefined && err.status === 500) {
        if (typeof err.responseJSON === 'object') {
            const error = err.responseJSON.error.errors[0];

            message += '<br>';
            message += '<br>Location: ' + error.message.place;
            message += '<br>Mcode: ' + error.message.mcode;
            message += '<br>Error: ' + error.message.message;

        } else {
            if (err.error !== undefined) {
                message += err.error.description

                if (err.error.dump !== undefined) {
                    message += '<br>'

                    err.error.dump.forEach(line => {
                        message += line + '<br>'
                    })
                }

            } else {
                message += 'Internal error ' + err.responseText
            }
        }
    } else if (err.error !== undefined) {
        message += err.error.description

        if (err.error.dump !== undefined) {
            message += '<br><br>'

            err.error.dump.forEach(line => {
                message += line + '<br>'
            })
        }
    } else {
        let error = err.statusText
        if (err.statusText === 'error' && err.status === 0) {
            error = 'Can not connect to the REST server...'
        }

        message += error
    }

    return message
};
