/****************************************************************
 *                                                              *
 * Copyright (c) 2022-2023 YottaDB LLC and/or its subsidiaries.      *
 * All rights reserved.                                         *
 *                                                              *
 * This source code contains the intellectual property          *
 * of its copyright holder(s), and is made available            *
 * under a license.  If you do not know the terms of            *
 * the license, please stop and do not read further.            *
 *                                                              *
 ****************************************************************/

let settings = {};

// *******************
// Initialization
// *******************

// System Initialization at startup
window.onload = async () => {
    // detect browser type
    let isSafari = window.safari !== undefined;

    if (isSafari === true) {
        alert('The browser Safari is not currently supported. Please use Chrome or Firefox')
    }

    // loads all included HTML
    await loadFiles('data-include');

    // initialize popper
    $('[data-toggle="popover"]').popover({
        html: true, container: 'body', template: '<div class="popover" role="tooltip"><div class="arrow "></div><h3 class="popover-header ydb-popover-header "></h3><div class="popover-body"></div></div>'
    });

    // detect storage operation RESET
    storageReset = window.location.search.indexOf('?storage=reset') > -1;

    // Init handlers
    app.ui.addHandlers();

    // detect testMode
    testMode = window.location.search.indexOf('?test=') > -1;

    // Load test file if needed
    if (testMode) {
        await loadTestScript()
    }

    // get YDBGUI version number
    try {
        $.getJSON('_ydbgui.manifest.json', json => {
            app.version = json.version;

            // Start
            app.ui.dashboard.refresh()
        })

    } catch (err) {
        console.log('The following error occurred while fetching the version.json file:');
        console.log(err)
    }

    window.addEventListener('beforeunload',  e => {
        if (app.ui.toaster.tasks > 0) {
            const str = 'There are tasks still running in the background...'

            e.preventDefault();
            e.returnValue = str

            return str
        }
    });
};

const loadFiles = async (tag) => {
    let promises = [];

    let includes = $('[' + tag + ']');
    jQuery.each(includes, async function () {
        let def = new $.Deferred();

        const newThis = this;
        let file = './html/' + $(newThis).data('include') + '.html';
        $(this).load(file, () => {
            def.resolve();
        });
        promises.push(def)
    });

    return $.when.apply(undefined, promises).promise();
};

const loadTestScript = () => {
    return new Promise(function (resolve) {
        $.ajax({
            url: 'test/mockData.js',
            dataType: "script",
            success: resolve,
        });
    });
};

// *******************
// Main app object
// *******************

let app = {
    libs: {},

    REST: {
        host: window.location.hostname,
        port: window.location.port,
        root: '/api/',
        username: '',
        password: ''
    },
    ui: {
        menu: {},

        timings: {},

        msgbox: {},
        inputbox: {},
        wait: {},
        createDir: {},
        dirSelect: {},

        dashboard: {},

        regionView: {},
        regionAdd: {
            name: {}
        },
        regionNames: {
            add: {},
            delete: {}
        },
        regionShared: {},
        regionCreateDbFile: {},
        regionSelect: {},
        regionDelete: {},
        regionExtend: {},
        regionEdit: {},
        regionJournalSwitch: {},
        regionFilename: {},
        regionJournalFiles: {},
        locksManager: {},
        globalsManager: {},
        defrag: {},
        backup: {},
        gViewer: {},
        rViewer: {
            select: {}
        },
        help: {},
        tabs: {},
        compact: {},
        integ: {},
        storageSettings: {},
        storageRegion: {},
        storageWizard: {},
        backupWizard: {},
        storage: {},
        systemInfo: {
            envVars: {}
        },
        deviceInfo: {},
        dirSelect: {},
        fileSelect: {},
        globalMappings: {},

        toaster: {}
    },
    system: {},

    config: {
        timeout: 3600
    },
    currMouseTime: 0,
    timeoutId: 0
};

