/****************************************************************
 *                                                              *
 * Copyright (c) 2022-2023 YottaDB LLC and/or its subsidiaries.      *
 * All rights reserved.                                         *
 *                                                              *
 * This source code contains the intellectual property          *
 * of its copyright holder(s), and is made available            *
 * under a license.  If you do not know the terms of            *
 * the license, please stop and do not read further.            *
 *                                                              *
 ****************************************************************/

app.ui.tabs.init = () => {
    app.ui.tabs.cloneAddTab();
};

app.ui.tabs.select = async (type, metadata = null, withPromise = false) => {
    const oldTab = $('#tab' + type);

    // select if exists
    if (oldTab.length) {
        oldTab.tab('show');
        return;
    }

    // or create a new one
    const tabType = type.split('-')[1];

    // remove the add tab
    $('#liTabsAdd').remove();

    // tab header
    $('#tabsContainer').append($('<li/>', {
        class: 'nav-item',
        role: 'presentation',
        id: 'liTabs' + type
    }));

    $('#liTabs' + type).append($('<a/>', {
        class: "nav-link tab-link inconsolata",
        id: "tab" + type,
        'data-toggle': "tab",
        href: "#" + (tabType === 'G' ? 'tabGviewer' : 'tabRviewer') + type + 'Div',
        role: "tab",
        'aria-controls': type,
        'aria-selected': "false",
    }));

    const newTab = $('#tab' + type);
    $('a[data-toggle="tab"]').on('hide.bs.tab', function (e) {
        $('#' + e.target.id + 'Div').css('display', 'none');
        $('#' + e.relatedTarget.id + 'Div').css('display', 'block');


    });

    const caption = '<span class="tab-pill">' + (metadata === null ? '&nbsp;G' : '&nbsp;R') + '&nbsp;' + '</span>&nbsp;' + (metadata === null ? 'New global' : metadata.routineName.replace(/_/g, "%"));

    newTab
        .html(caption + '&nbsp;&nbsp;&nbsp;<span onclick="app.ui.tabs.close(\'tab' + type + '\')" class="tab-close-button close" title="">x</span>')
        .attr('title', tabType === 'G' ? caption : '');

    // tab body
    if (tabType === 'G') {
        app.ui.gViewer.createInstance(type);
    } else {
        await app.ui.rViewer.createInstance(type, metadata);
    }

    newTab.addClass('tab-font');
    newTab.tab('show');

    app.ui.tabs.cloneAddTab();
    if (withPromise === true) {
        return new Promise(async function (resolve) {
            resolve();
        })
    }
};

app.ui.tabs.close = (tabId) => {
    const prevItem = $('#' + tabId).parent().prev();

    $('#' + tabId).parent().remove();
    $('#' + tabId + 'Div').remove();

    if (tabId.indexOf('G') > -1) {
        // clear the memory
        delete app.ui.gViewer.instance[tabId];

    } else {
        // clear the memory
        delete app.ui.rViewer.instance[tabId];
    }

    if (prevItem.length !== 0) {
        // select prev sibling tab
        prevItem.find('a').tab('show');

        let prevItemDivId = 'tab-' + prevItem.attr('id').split('-')[1] + '-' + prevItem.attr('id').split('-')[2] + 'Div';
        if (prevItem.attr('id') === 'liTabsDashboard') {
            prevItemDivId = 'tabDashboardDiv'
        }
        $('#' + prevItemDivId).css('display', 'block')
    }
};

app.ui.tabs.cloneAddTab = () => {
    const newTabAdd = $('#liTabsAddTemplate').clone();

    newTabAdd
        .attr('id', 'liTabsAdd')
        .css('display', 'block');

    $('#tabsContainer').append(newTabAdd)
};
