/****************************************************************
 *                                                              *
 * Copyright (c) 2022-2023 YottaDB LLC and/or its subsidiaries.      *
 * All rights reserved.                                         *
 *                                                              *
 * This source code contains the intellectual property          *
 * of its copyright holder(s), and is made available            *
 * under a license.  If you do not know the terms of            *
 * the license, please stop and do not read further.            *
 *                                                              *
 ****************************************************************/

// ************************************
// add handlers
// ************************************

app.ui.addHandlers = () => {
    // msgbox
    app.ui.msgbox.init();

    // input box
    app.ui.inputbox.init();

    // wait
    app.ui.wait.init();

    // help
    app.ui.help.init();

    // dashboard
    app.ui.dashboard.init();

    // systemInfo
    app.ui.systemInfo.init();

    //deviceInfo
    app.ui.deviceInfo.init();

    // region view
    app.ui.regionView.init();

    //region add
    app.ui.regionAdd.init();
    app.ui.regionAdd.name.init();
    app.ui.regionFilename.init();

    //region edit
    app.ui.regionEdit.init();
    app.ui.regionEdit.Overview.init();

    // region names
    app.ui.regionNames.add.init();

    // region delete
    app.ui.regionDelete.init();

    // region select
    app.ui.regionSelect.init();

    // region extend
    app.ui.regionExtend.init();

    // regionJournalFiles
    app.ui.regionJournalFiles.init();

    // regionCreateDbFile
    app.ui.regionCreateDbFile.init();

    // region defrag
    app.ui.defrag.init();

    // region integ
    app.ui.integ.init();

    // region backup
    app.ui.backup.init();

    // locksManager
    app.ui.locksManager.init();
    app.ui.locksManager.filter.init();

    // local storage
    app.ui.storage.init();

    // menu handlers
    app.ui.menu.init();

    // tabs
    app.ui.tabs.init();

    // dirSelect
    app.ui.dirSelect.init();

    // fileSelect
    app.ui.fileSelect.init();

    // globals
    app.ui.globalsManager.init();
    app.ui.gViewer.moreItems.init();
    app.ui.gViewer.settings.init();
    app.ui.gViewer.settings.confirm.init();

    // routines
    app.ui.rViewer.select.init();
    app.ui.rViewer.searchPath.init();
    app.ui.rViewer.searchPath.initAdd();

    // global Mapping
    app.ui.globalMappings.init();
};

// ************************************
// msgbox
// ************************************
app.ui.msgbox.init = () => {
    $('#btnMsgboxOk').on('click', (e) => app.ui.msgbox.hide());
    // TEST
    app.ui.setupDialogForTest('modalMsgbox')
    $('#modalMsgbox').on('shown.bs.modal', () => {
        $('#modalMsgbox').css('animation-name', 'shown')
        $('#btnMsgBoxOk').focus()
    });
};

app.ui.msgbox.show = (msg, title = '', withPromise = false) => {
    $('#txtMsgboxText').html(msg);
    $('#txtMsgboxTitle').text(title !== '' && title !== undefined ? title : 'WARNING');

    const msgboxHeader = $('#msgboxHeader');
    switch (title) {
        case 'ERROR':
        case 'FATAL': {
            msgboxHeader.css('background', 'var(--ydb-status-red)');
            msgboxHeader.css('color', 'white');
            break

        }
        case 'WARNING': {
            msgboxHeader.css('background', 'var(--ydb-status-amber)');
            msgboxHeader.css('color', 'var(--jhTertiary3)');
            break

        }
        default: {
            msgboxHeader.css('background', 'var(--ydb-status-green)');
            msgboxHeader.css('color', 'white');
        }
    }

    $('#modalMsgbox').modal({show: true, backdrop: 'static', keyboard: true});
    if (withPromise) {
        return new Promise(function (resolve) {
            $('#modalMsgbox').on('hide.bs.modal', () => {
                resolve()
            })
        })
    }
};

app.ui.msgbox.hide = () => {
    $('#modalMsgbox').modal({show: false});
};

// ************************************
// Inputbox
// ************************************

app.ui.inputbox.init = () => {
    $('#btnInputboxYes').on('click', (e) => app.ui.inputbox.yesSelected());
    $('#btnInputboxNo').on('click', (e) => app.ui.inputbox.noSelected());

    app.ui.setupDialogForTest('modalInputbox')
    $('#modalInputbox').on('shown.bs.modal', () => {
        $('#modalInputbox').css('animation-name', 'shown')
        $('#btnInputboxNo').focus()
    });
};

app.ui.inputbox.show = (msg, title, callback, options) => {
    options = options || {};

    if (options.buttons === undefined) {
        options.buttons = {
            left: 'Yes', right: 'No'
        }
    }

    const inputboxHeader = $('#inputboxHeader');
    switch (title) {
        case 'ERROR':
        case 'WARNING':
        case 'FATAL': {
            inputboxHeader.css('background', 'var(--ydb-status-red)');
            inputboxHeader.css('color', 'white');
            break

        }
        case 'Warning': {
            inputboxHeader.css('background', 'var(--ydb-status-amber)');
            inputboxHeader.css('color', 'var(--ydb-purple)');
            break

        }
        default: {
            inputboxHeader.css('background', 'var(--ydb-status-green)');
            inputboxHeader.css('color', 'white');
        }
    }

    $('#btnInputboxYes').html(options.buttons.left);
    $('#btnInputboxNo').html(options.buttons.right);
    $('#txtInputboxText').html(msg);
    $('#txtInputboxTitle').text(title !== '' && title !== undefined ? title : 'WARNING');

    app.ui.inputbox.callback = callback;

    $('#modalInputbox').modal({show: true, backdrop: 'static'});
};

app.ui.inputbox.hide = () => {
    $('#modalInputbox').modal({show: false});
};

app.ui.inputbox.yesSelected = () => {
    app.ui.inputbox.callback("YES")
};

app.ui.inputbox.noSelected = () => {
    app.ui.inputbox.callback("NO")
};

// ************************************
// Wait
// ************************************
app.ui.wait.init = () => {
    app.ui.setupDialogForTest('modalWait')
}

app.ui.wait.show = (message, abortCallback = false) => {
    return new Promise(function (resolve, reject) {
        const modalWait = $('#modalWait');
        $('#txtWaitText').text(message);

        $('#divWaitAbortButton').css('display', abortCallback === false ? 'none' : 'block')

        if (abortCallback !== false) {
            $('#btnWaitAbortButton').on('click', () => {
                app.ui.wait.hide();
                abortCallback()
            })
        }
        modalWait.modal({show: true, backdrop: 'static', keyboard: false});
        resolve();

        modalWait.on('shown.bs.modal', function (e) {
            resolve()
        });
    })
};

app.ui.wait.hide = () => {
    $('#modalWait').modal('hide');

};

// *************************************************
// Libraries
// *************************************************

app.ui.formatBytes = (a, b = 2) => {
    if (0 === a) return "0 Bytes";
    const c = 0 > b ? 0 : b, d = Math.floor(Math.log(a) / Math.log(1024));
    let value = parseFloat((a / Math.pow(1024, d)).toFixed(c)).toString().split('.');
    if (value[1] === undefined) value.push('00'); else if (value[1].length === 1) value[1] += '0';

    return value.join('.') + " " + ["bytes", "KiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB"][d]
};

app.ui.formatThousands = x => {
    //inserts a comma as thousands separator each 3 chars
    return x.toString().replace(/\B(?<!\.\d*)(?=(\d{3})+(?!\d))/g, ",");
};

app.ui.getKeyValue = (array, key) => {
    if (Array.isArray(array)) {
        const flag = array.find(el => {
            return !(el[key] === undefined)
        });

        if (flag !== undefined) {
            return flag[key]
        } else return ''
    } else return ''
};

app.ui.countChar = (string, char) => {
    const regex = new RegExp('[^' + char + ']', 'g');

    return string.replace(regex, '').length;
};

app.ui.nthIndex = (string, pattern, n) => {
    let L = string.length, i = -1;

    while (n-- && i++ < L) {
        i = string.indexOf(pattern, i);
        if (i < 0) break;
    }
    return i;
};

// TEST utils
app.ui.setupDialogForTest = dialogId => {
    $('#' + dialogId)
        .on('show.bs.modal', () => {
            $('#' + dialogId).css('animation-name', 'showing')
        })
        .on('shown.bs.modal', () => {
            $('#' + dialogId).css('animation-name', 'shown')
        })
        .on('hide.bs.modal', () => {
            $('#' + dialogId).css('animation-name', 'hiding')
        })
        .on('hidden.bs.modal', () => {
            $('#' + dialogId).css('animation-name', 'hidden')
        })
}

app.ui.setTestClosure = dialogId => {
    $('#' + dialogId).css('animation-name', 'shown')
}

app.ui.getEnvVar = envvar => {
    let val = null

    app.system.systemInfo.envVars.forEach(envVar => {
        if (envVar.name === envvar) {
            val = envVar.value
        }
    })

    return val
}

app.ui.copyToClipboard = (clipboardText, $button) => {
    console.log(location.protocol)
    if (location.protocol === 'https:') {
        navigator.clipboard.writeText(clipboardText)

        const oldText = $button.html()
        $button.html('Copied to clipboard')

        setTimeout(() => {
            $button.html(oldText)

        }, 1500, $button, oldText)

    } else {
        alert(clipboardText)
    }
}
