<!--
/****************************************************************
*                                                              *
* Copyright (c) 2022-2023 YottaDB LLC and/or its subsidiaries. *
* All rights reserved.                                         *
*                                                              *
* This source code contains the intellectual property          *
* of its copyright holder(s), and is made available            *
* under a license.  If you do not know the terms of            *
* the license, please stop and do not read further.            *
*                                                              *
****************************************************************/
-->


## TESTING

The testing is performed by the means of standard Java Script testing tools: chai and puppeteer.
While chai manages the tests, puppeteer provides a headless Chrome browser with DOM access, to inspect graphic elements.

We test Client and Server independently, in the following way:

### Client

The Client testing is performed by generating different mock data in the `app.system` object, thus emulating multiple server responses scenario's. Each test number generates a different response, stimulating the interface to react according, so that we can inspect the graphic elements and determine the validity of the test.

### Server

The Server testing is not using the interface at all and is fully focused on the response returned by the REST call. By modifying the database structure, using OS or YDB MUPIP calls, we alter the REST response, so that we can analyze it and determine the validity of the test.];


# **CLIENT**


#### Dashboard: gld file

| Number | Description |
| :---:| ---      |
| 30 | When gld file is missing|

#### Dashboard: Regions list: Database

| Number | Description |
| :---:| ---      |
| 36 | when region[0] has a valid file|
| 37 | when region[0] has a file missing + no auto db|
| 38 | when region[0] has a file missing + auto db|
| 39 | when region[0] has a file ok but shmem is bad|
| 40 | Extension count > 0: 1 extensions left|
| 41 | Extension count > 0: 4 extensions left|
| 42 | Extension count > 0: 9 extensions left|
| 43 | Extension count = 0: 10% of db size left|
| 44 | Extension count = 0: 15% of db size left|
| 45 | Extension count = 0: 25% of db size left|
| 46 | Freeze mode is on|
| 47 | Dashboard: When an interrupted REORG is detected|

#### Dashboard: Regions list: Journal

| Number | Description |
| :---:| ---      |
| 50 | when journal file is disabled AND repl off|
| 51 | when journal file is enabled but off AND repl off|
| 52 | when journal file is enabled and on (before) AND repl off|
| 53 | when journal file is enabled and on (nobefore) AND repl off|
| 54 | when journal file is enabled and in WasOn status|
| 55 | when journal file is enabled but off AND repl on AND 0 users|
| 56 | when journal file is enabled and on (before) AND repl on AND 0 users|
| 57 | when journal file is enabled and on (nobefore) AND repl on AND 0 users|
| 58 | when journal file is disabled AND repl on AND 0 users|
| 59 | when journal file is disabled AND repl on AND >0 users|
| 60 | when journal file is enabled but off AND repl on AND >0 users|
| 61 | when journal file is enabled and on (before) AND repl on AND >0 users|
| 62 | when journal file is enabled and on (nobefore) AND repl on AND >0 users|
| 63 | Journal file is missing and state = enabled/on|

#### Dashboard: Regions list: Devices

| Number | Description |
| :---:| ---      |
| 70 | When disk space is <70%|
| 71 | When disk space is >70% and <90%|
| 72 | When disk space is >90% and <97%|
| 73 | When disk space is >97% and <101%|

#### Dashboard: Global status

| Number | Description |
| :---:| ---      |
| 100 | When all dbs are ok|
| 101 | When a few db's have issues|
| 102 | When ALL dbs have issues|
| 103 | When all journals are ok|
| 104 | When some journals have issues|
| 105 | When all journals have issues|
| 106 | When at least one journal has the wasOn status set|
| 107 | When no region has replication turned on|
| 108 | When no region has replication turned on|
| 109 | When at least one journal has the wasOn status set|
| 110 | When freeze mode is on|
| 111 | Journal file is missing and state = enabled/on|
| 112 | If all regions have journal disabled, global status should be 'disabled'|

#### Dashboard: Events

| Number | Description |
| :---:| ---      |
| 120 | Clicking the region list[0] icon|
| 122 | Clicking the System info menu item|
| 123 | Clicking the devices[0] icon|
| 124 | Replication column when REPL = ON|
| 125 | Replication column when REPL = OFF|
| 126 | Replication column when REPL = WAS_ON|

#### Dashboard: host and replication instance name

| Number | Description |
| :---:| ---      |
| 130 | Ensure that the host name is displayed|
| 131 | Ensure that the Repl. Instance name reads: Not configured...|
| 132 | Ensure that the Repl. Instance name reads: DUMMY|
| 133 | Create an repl instance file, turn repl on and verify that repl instance name is populate, clean up when done |

#### Dashboard: RO / RW mode indicator

| Number | Description |
| :---:| ---      |
| 148 | Ensure that the text is correct|
 

#### System Info

| Number | Description |
| :---:| ---      |
| 150 | Confirm that the three labels are populated correctly|
| 151 | Verify the existence of one plugin|
| 152 | Clicking the Env Var button should open a dialog|
| 170 | Env Vars: confirm that the list is populate with all three categories|
| 171 | Env Vars: Confirm that the list is populate with only system info|
| 172 | Env Vars: Confirm that the list is populate with only YDB data|
| 173 | Env Vars: Confirm that the list is populate with only GTM data|
| 174 | Confirm that the encryption flag is populated correctly|
| 175 | Verify that manifest file for _ydbgui is correct|
 

#### Devices

| Number | Description |
| :---:| ---      |
| 190 | Confirm that the four labels are populated correctly|
| 191 | Gauge when disk space is <70%|
| 192 | Gauge when disk space is >70% and <90%|
| 193 | Gauge when disk space is >90% and <97%|
| 194 | Gauge when disk space is >97% and <101%|
| 195 | Ensure that the extra device information is properly displayed|
 

#### Devices

| Number | Description |
| :---:| ---      |
| 190 | Confirm that the four labels are populated correctly|
 

#### Dashboard: PULLDOWN MENUS

| Number | Description |
| :---:| ---      |
| 200 | Make at least one region with no file AND autodb = true and verify that create db menu is enabled|
| 201 | Remove all journals from all regions (state = 0) and verify that journaling on/off is disabled|
| 202 | If all no regions have a file, extend menu is disabled|
| 203 | Ensure the extend menu is disabled in RO mode|
| 204 | Ensure the journal menu is disabled in RO mode|
| 205 | Ensure the create database is disabled in RO mode|
| 206 | Ensure the create region menu is disabled in RO mode|
| 207 | Ensure the edit region menu is disabled in RO mode|
| 208 | Ensure the delete region menu is disabled in RO mode|
| 209 | Ensure the journal switch menu is disabled in RO mode|
 

#### Region View: Database tab

| Number | Description |
| :---:| ---      |
| 250 | Db status when db file is missing and autodb is on|
| 251 | Db status when db file is missing and autodb is off|
| 252 | Db status when db file is bad|
| 253 | Db status when shmem  is bad|
| 254 | blocks gauge when free block space is <70%|
| 255 | blocks gauge when free block space is >70% and <90%|
| 256 | blocks gauge when free block space is >90% and <97%|
| 257 | blocks gauge when free block space is >97% and <101%|
| 258 | User sessions when users = 0|
| 259 | User sessions when users > 0|
| 260 | Alert message when db file is missing and autodb is on|
| 261 | Alert message when db file is missing and autodb is off|
| 262 | Alert message when db file is bad|
| 263 | Alert message when shmem is bad|
| 270 | Db file: Verify that the table gets populated|
| 271 | Db Access lists: Verify that the table gets populated|
| 280 | Button Create db: verify that is disabled when db file is missing and filename is empty|
| 281 | Button Create db: verify that is enabled when db file is missing and filename is specified|
| 282 | Button Create db: verify that is disabled when db file is good|
| 283 | Button Extend db: verify that is enabled at all times when database file exists and it is valid|
| 284 | Button Extend db: verify that is disabled  when database file does NOT exists|
| 285 | Button Extend db: verify that is disabled  when database file is invalid|
| 286 | Select Advanced Params and verify that extra fields are displayed in the db access table|
| 287 | when db file >90 % and extension > 0 should be green|
| 288 | Extension count > 0: 1 extensions left|
| 289 | Extension count > 0: 5 extensions left|
| 290 | Extension count > 0: 8 extensions left|
| 291 | Extension count = 0: 10% of db size left|
| 292 | Extension count = 0: 15% of db size left|
| 293 | Extension count = 0: 25% of db size left|
| 294 | With extension > 0, verify that Db Usage background is always green, but fore changes at 35%|
| 295 | Verify that, in RO mode, create database button is disabled|
| 296 | Verify that, in RO mode, extend database button is disabled|
| 297 | Region view: When an interrupted REORG is detected|

#### Region View: Journal tab

| Number | Description |
| :---:| ---      |
| 300 | Status when journal file is disabled AND repl off|
| 301 | Status when journal file is enabled but off AND repl off|
| 302 | Status when journal file is enabled and on (before) AND repl off|
| 303 | Status when journal file is enabled and in WasOn status|
| 304 | Status when journal file is enabled but off AND repl on AND 0 users|
| 305 | Status when journal file is enabled and on (before) AND repl on AND 0 users|
| 306 | Status when journal file is enabled and on (nobefore) AND repl on AND 0 users|
| 307 | Status when journal file is disabled AND repl on AND >0 users|
| 308 | Status when journal file is enabled but off AND repl on AND >0 users|
| 310 | If no journal, turn on/off button should be invisible|
| 313 | Verify that Type pill is properly populated with the type when journaling is enabled and Before|
| 314 | Verify that Type pill is properly populated with the type when journaling is enabled and Nobefore|
| 315 | Verify that Type pill is properly populated with the type when journaling disabled|
| 316 | Verify that the Parameters list gets properly populated|
| 317 | Verify that the entire Journal Parameter list is hidden when journal is disabled|
| 318 | Alert message when WAS ON is set|
| 319 | Alert message when replication is on and journal is disabled|
| 320 | Alert message when replication is on and journal is enabled / off|
| 321 | Alert message when replication is off and journal is enabled / off|
| 322 | Journal file is missing and state = 2: verify alert|
| 323 | Journal file is missing and state = 2: verify button text|
| 324 | Verify that, in RO mode, turn on/off button is disabled|

#### Region View: Names tab

| Number | Description |
| :---:| ---      |
| 350 | Verify that the list is empty|
| 351 | Verify that the list content changes when clicking the checkbox and display also the %YDBOCTO|
| 352 | Ensure, when choosing a tab different than Region or Journal, that checkbox \"Advanced\" is hidden|

#### Region View: Stats tab

| Number | Description |
| :---:| ---      |
| 360 | Verify that the table gets populated with >0 items|

#### Region View: Locks tab

| Number | Description |
| :---:| ---      |
| 370 | Verify that the label Processes on queue gets populated|
| 371 | Verify that the label Lock slots in use gets populated|
| 372 | Verify that the label Slot bytes in use gets populated|
| 373 | Verify that the label Free lock space gets populated|
| 374 | Lock table: no locks|
| 375 | Lock table: 1 lock, no waiter|
| 376 | Lock table: 1 lock, 1 waiter|
| 377 | Lock table: 1 lock, 2 waiters|
| 378 | Lock table: 2 lock, no waiter|
| 379 | Lock table: 2 locks, 1 waiter|
| 380 | Lock table: 2 locks, 2 waiters|
 

#### Region Delete

| Number | Description |
| :---:| ---      |
| 400 | Verify that clicking YES it will display a second dialog|
| 401 | Delete a region, type in the wrong region name and verify that you can NOT submit|
| 402 | Delete a region, type in the correct region and verify that you can submit|
 

#### Region Extend

| Number | Description |
| :---:| ---      |
| 410 | Verify that the dialog is displayed|
| 411 | Verify that the top three labels are populated correctly|
| 412 | Based on entered ext. size, ensure that New size and New avail. space are correct|
| 413 | Submit the form and expect a dialog|
| 414 | Submit the form and answer NO. Expect the dialog to disappear|
| 415 | Submit the form and answer YES. Expect the dialog to disappear|
 

#### Region Select

| Number | Description |
| :---:| ---      |
| 430 | Display dialog, scroll down twice, submit: verify that the correct dialog is open|
| 431 | Display dialog, scroll down twice and then up, submit: verify that the correct dialog is open|
 

#### Create Database

| Number | Description |
| :---:| ---      |
| 440 | Verify that dialog is displayed|
| 441 | Verify that the three labels are populated correctly|
| 442 | Submit the form and expect a dialog|
| 443 | Submit the form and answer NO. Expect the dialog to disappear|
| 444 | Submit the form and answer YES. Expect the dialog to close and an error dialog open with error: file already exists|
 

#### Turn journal on/off dialog

| Number | Description |
| :---:| ---      |
| 450 | Verify that the dialog is displayed and with the text 'OFF'|
| 451 | Submit the form and answer NO. Expect the dialog to close|
| 452 | Submit the form and answer YES. Expect the dialog to close|
 

#### Switch Journal dialog

| Number | Description |
| :---:| ---      |
| 450 | Verify that the dialog is displayed and with the text 'OFF'|
| 451 | Submit the form and answer NO. Expect the dialog to close|
| 452 | Submit the form and answer YES. Expect the dialog to close|
 

#### Add Region: Region tab

| Number | Description |
| :---:| ---      |
| 490 | Default display should be BG table|
| 491 | Clicking MM it should display the MM table|
| 492 | Click on BG filename and verify dialog is displayed|
| 493 | Click on MM filename and verify dialog is displayed|
| 494 | Click on advanced mode and verify that BG table has more rows|
| 495 | Click on advanced mode and verify that MM table has more rows|
| 496 | Click on advanced mode and verify that right table has more rows|

#### Add Region: Journal tab

| Number | Description |
| :---:| ---      |
| 520 | Default display should be no journal|
| 522 | Turn the journal on and verify that table is displayed|
| 523 | Click on Journal filename and verify dialog is displayed|
| 524 | Default display should be no journal|

#### Add Region: Names tab

| Number | Description |
| :---:| ---      |
| 540 | Click on add name button and verify that dialog is displayed|
| 541 | Click on add name button and verify that dialog is displayed|
| 542 | Add a name, select it, click delete name and verify that it gets deleted|
| 543 | Add multiple names and ensure they got sorted|
 

#### Edit Region: Region tab

| Number | Description |
| :---:| ---      |
| 555 | Edit DEFAULT and verify that BG is checked|
| 556 | Edit DEFAULT and switch to MM: Edit button should be enabled|
| 557 | Edit YDBAIM and verify that MM is checked|
| 558 | Edit YDBAIM and switch to BG: Edit button should be enabled|
| 559 | Click Advanced parameters and verify that left table is properly populated|
| 560 | Click Advanced parameters and verify that right table is properly populated|
| 561 | Edit DEFAULT and click on the filename button. Verify that the message box appears|
| 562 | Edit YDBAIM and click on the filename button. Verify that the message box appears|

#### Edit Region: Journal tab

| Number | Description |
| :---:| ---      |
| 570 | Edit DEFAULT and verify that journal is ON|
| 571 | Edit DEFAULT and switch the journal off. Verify that the table gets hidden|
| 572 | Edit DEFAULT and switch the journal off. Verify that the Edit button is enabled|
| 573 | Edit DEFAULT and click on the filename button. Verify that the Filename dialog appears|

#### Edit region: Names Tab

| Number | Description |
| :---:| ---      |
| 585 | Edit DEFAULT, add a name TEST and verify that appears in the list as GREEN|
| 586 | Edit YDBOCTO, add a name %a and verify that appears in the list as first and in GREEN color|
| 587 | Edit YDBOCTO, add a name %b, select it and verify that delete button caption is: Delete...|
| 588 | Edit YDBOCTO, select the first entry and delete it. Verify that text has the RED color|
| 589 | Edit YDBOCTO, select the first entry and delete it. Verify that delete button caption is: Undelete...|
| 590 | Edit YDBOCTO, select the first entry and delete it. Click the Undelete... button and verify that text is in Purple color|
 

#### Tree integrity

| Number | Description |
| :---:| ---      |
| 600 | One lock per region with different PIDs, 1, 2 and 3 waiters per lock. Test By Namespace tree, lock 1|
| 601 | One lock per region with different PIDs, 1, 2 and 3 waiters per lock. Test By Namespace tree, lock 2|
| 602 | One lock per region with different PIDs, 1, 2 and 3 waiters per lock. Test By Namespace tree, lock 3|
| 603 | One lock per region with different PIDs, 1, 2 and 3 waiters per lock. Test By Region tree, lock 1|
| 604 | One lock per region with different PIDs, 1, 2 and 3 waiters per lock. Test By Region tree, lock 2|
| 605 | One lock per region with different PIDs, 1, 2 and 3 waiters per lock. Test By Region tree, lock 3|
| 606 | One lock per region with different PIDs, 1, 2 and 3 waiters per lock. Test By PIDs tree, lock 1|
| 607 | One lock per region with different PIDs, 1, 2 and 3 waiters per lock. Test By PIDs tree, lock 2|
| 608 | One lock per region with different PIDs, 1, 2 and 3 waiters per lock. Test By PIDs tree, lock 3|
| 609 | Test sort on namespace|
| 610 | Test sort on region|
| 611 | Test sort on PIDs|
| 612 | No locks at all. Test By Namespace tree|
| 613 | No locks at all. Test By Regions tree|
| 614 | No locks at all. Test By PIDs tree|

#### Submit messages => Clear lock

| Number | Description |
| :---:| ---      |
| 615 | Test By Namespace tree, select namespace, click on Clear Lock|
| 616 | Test By Namespace tree, select region, click on Clear Lock|
| 617 | Test By Namespace tree, select PID, click on Clear Lock|
| 618 | Test By Namespace tree, select first waiter, click on Clear Lock|
| 619 | Test By Region tree, select region, click on Clear Lock|
| 620 | Test By Region tree, select namespace, click on Clear Lock|
| 621 | Test By Region tree, select pid, click on Clear Lock|
| 622 | Test By Region tree, select first waiter, click on Clear Lock|
| 623 | Test By Pids tree, select pid, click on Clear Lock|
| 624 | Test By Pids tree, select namespace, click on Clear Lock|
| 625 | Test By Pids tree, select region, click on Clear Lock|
| 626 | Test By Pids tree, select first waiter, click on Clear Lock|
| 640 | Verify that, in RO mode, clear lock button is disabled|

#### Submit messages => Terminate process

| Number | Description |
| :---:| ---      |
| 627 | Test By Namespace tree, select namespace, click on Terminate process|
| 628 | Test By Namespace tree, select region, click on Terminate process|
| 629 | Test By Namespace tree, select PID, click on Terminate process|
| 630 | Test By Namespace tree, select first waiter, click on Terminate process|
| 631 | Test By Region tree, select region, click on Terminate process|
| 632 | Test By Region tree, select namespace, click on Terminate process|
| 633 | Test By Region tree, select pid, click on Terminate process|
| 634 | Test By Region tree, select first waiter, click on Terminate process|
| 635 | Test By Pids tree, select pid, click on Terminate process|
| 636 | TestTest By Pids tree, select namespace, click on Terminate process|
| 637 | Test By Pids tree, select region, click on Terminate process|
| 638 | Test By Pids tree, select first waiter, click on Terminate process|
| 639 | Verify that, in RO mode, terminate process button is disabled|

#### Filtering

| Number | Description |
| :---:| ---      |
| 650 | Ensure the info bar is correct without any filter|
| 651 | Display the filter dialog |
| 652 | Display the filter dialog, type a filter and submit, verify info bar |
| 653 | Submit a filter, reset the filter and verify the info bar |
| 654 | Verify that Reset Filter button is disabled|
 

#### Globals > Tabs management

| Number | Description |
| :---:| ---      |
| 700 | Create a new tab: verify default text|
| 705 | Create 1 new tab: verify that you can switch tabs using the menu|
| 706 | Create a new tab with very long name: verify that caption gets truncated as \"settings - maxLengthCaption\" and popup is correct|
| 707 | Create a new tab with long name and change the \"settings - maxLengthCaption\" : verify that caption gets truncated as \"settings - maxLengthCaption\" and popup is correct|

#### Globals > Menu management

| Number | Description |
| :---:| ---      |
| 750 | Verify that the Last Used list gets populated with default # of items|
| 751 | Verify that the Last Used list gets populated with different # of items && more is displayed|
| 752 | Verify that the More dialog is properly populated|
| 754 | Type a valid path and verify that gets added to the list|
| 755 | Type an existing path and verify that it doesn't get added to the list|
 

#### Parser > Global

| Number | Description |
| :---:| ---      |
| 800 | Test text: ^ only: error|
| 805 | Test text: ^%v0.1 error|
| 806 | Test text: ^%a%b error|
| 807 | Test text: ^TE%ST error|
| 808 | Test text: ^a123456789012345678901234567890abc error|
| 809 | Test text: ^a123456789012345678901234567890 good|
| 810 | Test text: ^AA$% error|
| 811 | Test text: ^a@ error|
| 812 | Test text: ^A: error|
| 813 | Test text: ^notexist error|
| 814 | Create global ^TEST Test text: ^TEST good|
| 815 | Create global ^TEST Submit text using button: ^TEST good|
| 816 | Submit text using button: ^% good|
| 817 | Submit text using button: ^%test good|

#### Parser > Parens

| Number | Description |
| :---:| ---      |
| 850 | Test text: ^( error|
| 851 | Test text: ^(( error|
| 852 | Test text: ^) error|

#### Parser > Subscripts > Commas and stars

| Number | Description |
| :---:| ---      |
| 870 | Test text: ^TEST(*) good|
| 871 | Test text: ^TEST(,*) good|
| 872 | Test text: ^TEST(,,*) good|
| 873 | Test text: ^TEST(,,,) good|
| 874 | Test text: ^TEST(,) good|
| 875 | Test text: ^TEST(*,23) error|
| 924 | Test text: ^TEST(23 bad|

#### Parser > Subscripts > Numeric

| Number | Description |
| :---:| ---      |
| 900 | Test text: ^TEST(100) good|
| 901 | Test text: ^TEST(100,100) good|
| 902 | Test text: ^TEST(-100) good|
| 903 | Test text: ^TEST(100.23) good|
| 904 | Test text: ^TEST(.100) good|
| 905 | Test text: ^TEST(100e24) good|
| 906 | Test text: ^TEST(100E24) good|
| 907 | Test text: ^TEST(-100.2) good|
| 908 | Test text: ^TEST(-.100) good|
| 909 | Test text: ^TEST(100,) good|
| 910 | Test text: ^TEST(100,*) good|
| 911 | Test text: ^TEST(100,) good|
| 912 | Test text: ^TEST(100,23) good|
| 913 | Test text: ^TEST(100,45,,,) good|
| 914 | Test text: ^TEST(100,23,*) good|
| 915 | Test text: ^TEST(100,,23) good|
| 916 | Test text: ^TEST(100,,23,*) good|
| 917 | Test text: ^TEST(23e12e12) bad|
| 918 | Test text: ^TEST(23e) bad|
| 919 | Test text: ^TEST(23.12.34) bad|
| 920 | Test text: ^TEST(23.12.) bad|
| 921 | Test text: ^TEST(2312-3) bad|
| 922 | Test text: ^TEST(23Ee12e12) bad|
| 923 | Test text: ^TEST(23E) bad|
| 924 | Test text: ^TEST(23 bad|

#### Parser > Subscripts > String

| Number | Description |
| :---:| ---      |
| 950 | Test text: ^TEST(\"100\") good|
| 951 | Test text: ^TEST(\"abc\") good|
| 952 | Test text: ^TEST(\"1\"0\") error|
| 953 | Test text: ^TEST(\"a\"c\"100\") error|
| 954 | Test text: ^TEST(\"%$#^%^&*\") good|
| 955 | Test text: ^TEST(\"100\",*) good|
| 956 | Test text: ^TEST(\"100\",\"test\") good|
| 957 | Test text: ^TEST(\"100\",,\"test\") good|
| 958 | Test text: ^TEST(\"100\",,,\"test\") good|
| 959 | Test text: ^TEST(\"100\",,,\"test\",*) good|
| 960 | Test text: ^TEST(\"abc bad|
| 961 | Test text: ^TEST(\"abc) error|
| 962 | Test text: ^TEST(\"abc:\") good|

#### Parser > Subscripts > Ranges

| Number | Description |
| :---:| ---      |
| 1000 | Test text: ^TEST(:,:,*) good|
| 1001 | Test text: ^TEST(:23) good|
| 1002 | Test text: ^TEST(:\"abc\") good|
| 1003 | Test text: ^TEST(123:) good|
| 1004 | Test text: ^TEST(:,:,\"abc\":) good|
| 1005 | Test text: ^TEST(:,:,*) good|
| 1006 | Test text: ^TEST(\"abc\":\"def\",23,\"B\",*) good|
| 1007 | Test text: ^TEST(123:456,23,\"B\",*) good|
| 1008 | Test text: ^TEST(123:\"abc\",23,\"B\",*) good|
| 1009 | Test text: ^TEST(\"abc\":456,23,\"B\",*) good|
| 1010 | Test text: ^TEST(12:23,56:48,23,\"B\",*) good|
| 1011 | Test text: ^TEST(\"abc\":\"def\",12:34,23,\"B\",*) good|
| 1012 | Test text: ^TEST(:233,\"a\":,23,\"B\",*) good|
| 1013 | Test text: ^TEST(:233,*) good|
| 1014 | Test text: ^TEST(233:,*) good|
| 1015 | Test text: ^TEST(233:,:,*) good|
| 1016 | Test text: ^TEST(233:,:33,*) good|

#### Parser > Verify payload

| Number | Description |
| :---:| ---      |
| 1050 | Test text: ^TEST|
| 1051 | Test text: ^TEST(*)|
| 1052 | Test text: ^TEST(,)|
| 1053 | Test text: ^TEST(,*)|
| 1054 | Test text: ^TEST(12,,23,*)|
| 1055 | Test text: ^TEST(12,,,*)|
| 1056 | Test text: ^TEST(12,\"test\",,*)|
| 1057 | Test text: ^TEST(12:,:33,,\"a\",*)|
| 1058 | Test text: ^TEST(\"test\":\"zzz\",*)|
| 1059 | Test text: ^TEST(\"test\":\"zzz\",,*)|
| 1060 | Test text: ^TEST(100,23456,0)|
| 1061 | Test text: ^TEST(100,,23456,,0)|
| 1062 | Test text: ^TEST(:100,23456:,0)|
| 1063 | Test text: ^TEST(100,,23456,,0,*)|
| 1064 | Test text: ^TEST(,0)|
| 1065 | Test text: ^TEST(,0,*)|
 

#### Reorg

| Number | Description |
| :---:| ---      |
| 1100 | display dialog, change settings, close and reload again, verify default values are used|
| 1101 | Select the DEFAULT region, switch to Reclaim space and select the MM region, should display a message and set the switch off|
| 1102 | Select all regions and switch to Reclaim space, should display a message and set the switch off|
| 1104 | Deselect all regions, OK button should be disabled|
| 1005 | Change the range in the fillFactor, label should change|
| 1006 | Change the range in the indexFillFactor, label should change|
| 1007 | Select reclaim space, verify that range is visible, change the range in the Minimum amount for the reclamation, label should change|
| 1108 | Submit an operation. It should display an Inputpbox. Clicking cancel it should close it|
| 1109 | Reports on YDBOCTO and YDBAIM: at first it should display \"No globals\" on all regions|
| 1110 | Generate a report on DEFAULT, it should display values|
| 1111 | Generate a report on DEFAULT, generate a report, click on expand button and analyze the content|
| 1112 | set pendingReorg flag to true on DEFAULT, it should display the Pending dialog|
| 1113 | set pendingReorg flag to true on DEFAULT, the dialog should display a list with ONLY the faulty region|
| 1114 | set pendingReorg flag to true on DEFAULT, select NO, it should display a standard REORG dialog|
| 1115 | set pendingReorg flag to true on DEFAULT, select 'YES', it should display a reduced form with only the selected region, the resume switch and the background switch|
| 1116 | Generate a report on `DEFAULT`with recover space, it should display values|
| 1117 | Ensure \"Run background\" is reset after dialog close / open|
| 1118 | Submit using \"Run background\": message box should display extra text|
| 1119 | Run using \"Run background\": toaster should appear|
| 1120 | Run using \"Run background\": click on toast, should display result dialog|
| 1130 | RO mode: Menu entry should be disabled|
 

#### Globals > Table: general

| Number | Description |
| :---:| ---      |
| 1200 | Submit a query to ^ORD and verify that the record size is correct|
| 1201 | Data view: regular: Submit a query to ^ORD and verify that data is correct|
| 1202 | Data view: pieces: Submit a query to ^ORD and verify that data is correct|
| 1204 | Data view: pieces: Change piece char, submit a query to ^ORD and verify that data is correct|
| 1205 | Data view: pills: Change piece char, submit a query to ^ORD and verify that data is correct|
| 1206 | Data view: force the data size to 16 chars verify that data length gets trimmed|
| 1207 | Data view: force the data size to 16 chars verify that a button is present|
| 1208 | switch to dark mode and verify that table changes appearance|
| 1209 | Zoom in and verify that font is bigger|
| 1210 | Zoom out and verify that font is smaller|
| 1211 | Verify that the default bookmark color is loaded|
| 1212 | Verify that the default data view is loaded|
| 1213 | Verify that the default piece char is loaded|
| 1214 | Verify that the default theme is loaded|

#### Globals > Table: fetches

| Number | Description |
| :---:| ---      |
| 1250 | Set the max buffer to 100, initial fetch to 50: verify that data size = 50|
| 1251 | Set the max buffer to 100, initial fetch to 50 and lazy to 25: verify that, scrolling down, it is fetching|
| 1252 | Set the max buffer to 100, initial fetch to 50 and lazy to 25: verify that, scrolling down after buffer is full , it is fetching and beheading the buffer|
| 1253 | Set the max buffer to 100, initial fetch to 50 and lazy to 25: verify that, scrolling down after buffer is full and up again, it is fetching and betailing the buffer|
 

#### Globals > Editor: first tab: Global save to global

| Number | Description |
| :---:| ---      |
| 1300 | Change ALL Data buffer parameters, apply them to the current tab and inspect the class to verify|
| 1301 | Change the Last Used Count parameter, apply and verify that the menu size has changed|
| 1302 | Change the default: Piece char, apply and verify that the new tab gets the new value|
| 1303 | Change the default: Piece View, apply and verify that the new tab gets the new value|
| 1304 | Change the default: Theme, apply and verify that the new tab gets the new value|
| 1305 | Change the default: Bookmark color, apply and verify that the new tab gets the new value|

#### Globals > Editor: first tab: Path bar as global

| Number | Description |
| :---:| ---      |
| 1310 | Change the path bar: Global, apply and verify that the new tab gets the new value|
 

#### Menus

| Number | Description |
| :---:| ---      |
| 1500 | Select menu and verify that routines selector is displayed|

#### Routines select

| Number | Description |
| :---:| ---      |
| 1530 | Verify that query submit button is disabled|
| 1531 | Verify that open button is disabled|
| 1532 | Type: a and verify that submit is disabled|
| 1533 | Type: % and verify that submit is disabled|
| 1534 | Type: * and verify that submit is disabled|
| 1535 | Type: a* and verify that submit is disabled|
| 1536 | Type: %* and verify that submit is disabled|
| 1537 | Type: %y* and verify that submit is enabled, but open is disabled|
| 1538 | Type: %y* and verify that list is populated correctly|
| 1539 | Type: norou* and verify that list returns: No routines found...|
| 1540 | Type: %y*  and select one item, submit and verify that open is enabled|
| 1541 | Type: %y* and select two items, submit and verify that open is enabled|
| 1542 | Type: %y*  and select one item, submit and verify that one tab gets created|
| 1543 | Type: %y* and select two items, submit and verify that two tabs gets created|
 

#### Journal files

| Number | Description |
| :---:| ---      |
| 1700 | Display the dialog, it should list only the active journal file|
| 1701 | Switch journal files, it should list the active and the extra files in the chain|
| 1702 | Break the journal chain, it should display the active and the orphans|
| 1703 | Fill up the journal to trigger a switch, it should display the active file, the extra journal in the chain and the orphans|
| 1704 | Click on View details..., it should display the journal details dialog|
| 1705 | Click on View details..., ensure all entries are correct|

#### Switch Journal File

| Number | Description |
| :---:| ---      |
| 1720 | Click on the `Switch Journal` button, ensure inputbox is displayed|
| 1721 | Click on the Switch Journal button, cancel should close it|
| 1722 | Click on the Switch Journal button, select Yes, it should close it|
 

#### Integ

| Number | Description |
| :---:| ---      |
| 1600 | RO mode: Menu entry should be disabled|
| 1601 | Display dialog: ok button should be disabled|
| 1602 | Display dialog: select region, ok button should be enabled|
| 1603 | Display dialog: select region, deselect, ok button should be disabled|
| 1604 | Select type: fast'. Reporting detailed` should be disabled|
| 1605 | Select type: fast and then complete. Reporting detailed should be enabled|
| 1606 | select Advanced option, options should be visible|
| 1607 | select Advanced option twice, options should be hidden|
| 1608 | change all options on dialog, close and open up again, everything should be reset|
| 1609 | Select DEFAULT region, press Ok, inputbox should be displayed|
| 1610 | Select DEFAULT region, press Ok, inputbox should be displayed, press cancel, should close|
| 1611 | Select DEFAULT region, press Ok, inputbox should be displayed, press Ok, should generate report complete, summary|
| 1612 | Ensure \"Run background\" is reset after dialog close / open|
| 1613 | Submit using \"Run background\": message box should display extra text|
| 1614 | Run using \"Run background\": toaster should appear|
| 1615 | Run using \"Run background\": click on toast, should display result dialog|
| 1616 | Select DEFAULT region, press Ok, inputbox should be displayed, press Ok, should generate report complete, summary|

#### Backup

| Number | Description |
| :---:| ---      |
| 1650 | Display dialog, ensure all default values are correct|
| 1651 | Change all values, close dialog, reopen it and expect all default values correct|
| 1652 | Select DEFAULT, Ok button should be enabled|
| 1653 | Enter / as target directory, Validate should be enabled|
| 1654 | Enter / as target directory, press validate, it should be valid|
| 1655 | Enter / as target directory, press validate, ok button should be disabled|
| 1656 | Enter / as target directory, press validate, select DEFAULT, ok button should be enabled|
| 1658 | Select Create New Journal Files: Yes, link,  SyncIO should be enabled|
| 1659 | Select Create New Journal Files: Yes, nolink,  SyncIO should be enabled|
| 1660 | Select Create New Journal Files: No,  SyncIO should be disabled|
| 1661 | Mark DEFAULT as repl and clear repl env vars, select DEFAULT, msgbox should appear|
| 1662 | Mark DEFAULT as repl, select DEFAULT, msgbox should appear|
| 1663 | Mark DEFAULT as repl, create file, select DEFAULT, repl options should be visible|
| 1664 | Mark DEFAULT as repl, create file, select DEFAULT, validate path, repl options should be visible and ok button should be enabled|
| 1665 | Mark DEFAULT as repl, create file, select DEFAULT, uncheck Backup Replication Instance, other repl options should be invisible|
| 1666 | Mark DEFAULT as repl, create file, select DEFAULT, uncheck Backup Replication Instance, ok button should be enabled|
| 1667 | Mark DEFAULT as repl, create file, select DEFAULT, select Specify a Different Directory, extra fields should be visible|
| 1668 | Mark DEFAULT as repl, create file, select DEFAULT, select Specify a Different Directory, ok button should be enabled|
| 1669 | Mark DEFAULT as repl, create file, select DEFAULT, select Specify a Different Directory, validate should be disabled, type in / and validate should be enabled|
| 1670 | Mark DEFAULT as repl, create file, select DEFAULT, select Specify a Different Directory, validate should be disabled, type in /, validate: it should validate ok|
| 1671 | Mark DEFAULT as repl, create file, select DEFAULT, select Specify a Different Directory, validate should be disabled, type in /, validate: ok button should be enabled|
| 1672 | Select DEFAULT, enter / in path and validate, submit, msgbox should appear|
| 1673 | Select DEFAULT, enter / in path and validate, submit, msgbox should appear, cancel, should close|
| 1674 | Select DEFAULT, enter / in path and validate, submit, msgbox should appear, submit, should display the result dialog|
| 1675 | Select DEFAULT, enter / in path and validate, submit, msgbox should appear, submit, should check the result dialog for correctness|
| 1676 | click on ..., it should display the Dir Select dialog and have the path set to /|
| 1677 | type $ydb_dir, click on ..., it should display the Dir Select dialog and have the path set to /data|
| 1678 | type $ydb_dir, click on validate, should be ok|
| 1679 | type /notexist, click on validate, should be error|
| 1680 | type /YDBGUI/wwwroot/index.html, validate, should return error|
| 1681 | RO mode, should be disabled|
| 1682 | Ensure \"Run background\" is reset after dialog close / open|
| 1683 | Submit using \"Run background\": message box should display extra text|
| 1684 | Run using \"Run background\": toaster should appear|
| 1685 | Run using \"Run background\": click on toast, should display result dialog|

#### escribe("Toaster

| Number | Description |
| :---:| ---      |
| 1750 | Submit a REORG, check if the Async pill is visible, wait for reorg to complete, should disappear|
 

#### Toaster

| Number | Description |
| :---:| ---      |
| 1750 | Submit a REORG, check if the Async pill is visible, wait for reorg to complete, should disappear|
 

#### searchPath

| Number | Description |
| :---:| ---      |
| 1760 | Display the dialog|
| 1761 | Ensure that the tree is properly populated|
| 1762 | Ensure first item is selected: check status of all buttons|
| 1763 | Select source path: check status of all buttons|
| 1764 | Select System SO: check status of all buttons|
| 1765 | Select first entry (OBJ): move down: confirm|
| 1766 | Select second entry (SRC): move down: confirm|
| 1767 | Select 2nd object: move up: confirm|
| 1768 | Select 2nd item (first source): DELETE: confirm only source is deleted|
| 1769 | Select 4th item (2nd object): DELETE: confirm both obj and source are deleted|
| 1770 | Display dialog, select Add object path, should display dialog with Auto-relink option visible, Validate disabled, ok disabled and ... enabled|
| 1771 | Display dialog, select Add object path, type / as path and type validate, should validate ok and ok enabled|
| 1772 | Display dialog, select Add object path, type / as path and type validate, should validate ok and ok enabled, then type /123, ok should be disabled|
| 1773 | Display dialog, select Add object path, type /123 and validate, should display msgbox|
| 1774 | Display dialog, select Add object path, type /123 and ..., should display msgbox|
| 1775 | Display dialog, select Add object path, type / and ..., should display dir selector|
| 1776 | Display dialog, select Add object path, press ..., should display dir selector and default to /|
| 1777 | Display dialog, select Add object path, type / and validate, type ok, verify it has been added|
| 1778 | Display dialog, select Add object path, type / and validate, check the Auto-relink, type ok, verify it has been added with the * after the path|
| 1779 | Display dialog, select Add object path, type /YDBGUI/objects and validate, should display a message saying it already exists|
| 1780 | Display dialog, select Add source path, should display dialog with Validate disabled, ok disabled and ... enabled and no Auto-relink visible|
| 1781 | Display dialog, select Add source path, type / and validate, type ok, verify it has been added|
| 1782 | Display dialog, select Add source path, type /YDBGUI/routines and validate, type ok, should display a message saying that it already exists|
| 1783 | Display dialog, select Add library, should display dialog with Validate disabled, ok disabled and ... enabled|
| 1784 | Display dialog, select Add library, type /, click validate, should display a dialog saying that it is not a file|
| 1785 | Display dialog, select Add library, type /, click ..., should display the fileSelect dialog|
| 1786 | Display dialog, select Add library, type '/opt/yottadb/current/libgtmshr.so', validate, should validate ok|
| 1787 | Display dialog, select Add library, type '/YDBGUI/package.json', validate, should return message that it is not an .so file|
| 1788 | Display dialog, select Add library, type '/', validate, should return message that it is not an .so file|
| 1789 | Display dialog, select Add library, type '/opt/yottadb/current/libgtmshr.so', validate, select OK and verify that has been added|
| 1790 | Display dialog, select Add library, type '/opt/yottadb/current/libgtmshr.so', validate, select OK. Verify that buttons are correct|
| 1791 | Display dialog, select Add library, type '/opt/yottadb/current/libgtmshr.so', validate, select OK. Move it up|
| 1792 | Display dialog, select Add library, type '/opt/yottadb/current/libgtmshr.so', validate, select OK. Move it down once|
| 1793 | Display dialog, select Add library, type '/opt/yottadb/current/libgtmshr.so', validate, select OK. Move it down until the System SO, button down should be disabled|
| 1794 | Display dialog, select Add library, type '/YDBGUI/package.json', type ..., should display the dialog with path set to /YDBGUI|
| 1795 | Perform one add, then delete, then select ok: should display alert message: Nothing changed|
| 1796 | Add one object path, submit and check the inputbox text for correctness|
| 1797 | Add one object path, submit and check the inputbox text for correctness, re-open the window and confirm that the new path is displayed|
| 1798 | Create a routine bbb.m in /, add the path, perform a routine search, it should be found|
| 1799 | Create a routine bb.m in /, compile it as object in /, add the object, perform a routine search, it should be found, but with no source|
| 1800 | Create a routine bbb.m in /, compile it as object with the -emebed_source switch in /, add the object, perform a routine search, it should be found and ready to edit|
| 1801 | add the object dir '/' (as 2nd entry), copy the object bbb.o in /YDBGUI/objects, perform a routine search, it should be found in /YDBGUI/objects|
| 1802 | add the object dir '/' (as 1st entry), copy the object bbb.o in /YDBGUI/objects, perform a routine search, it should be found in /|
| 1803 | Display dialog, select Add object and source path, type / and validate, type ok, verify it has been added|
| 1804 | Display dialog, select Add object and source path, type / and validate, type ok, verify all buttons|
 

#### Global mappings

| Number | Description |
| :---:| ---      |
| 1820 | display dialog, ensure list is correct|
| 1821 | display dialog, ensure list contains *|
 
# **SERVER**


#### Endpoints verification

| Number | Description |
| :---:| ---      |
| 5000 | dashboard/getAll|
| 5001 | dashboard/regions/DEFAULT|
| 5002 | Ensure the extra device information is returned|
| 5020 | Use a wrong REST path and ensure that a 404 is returned|
| 5021 | Trigger an error in the server on GET and verify that an error is returned|
| 5022 | Trigger an error in the server on POST and verify that an error is returned|
| 5023 | Trigger an error in the server on DELETE and verify that an error is returned|
| 5024 | Rename a db file and verify that segment fields get populated anyway (with GDE data)|
| 5025 | Rename a db file and verify that region fields get populated anyway (with GDE data)|
| 5026 | Rename a db file and verify that journal fields get populated anyway (with GDE data)|
| 5027 | Create region with different segment name, pull the data and verify, delete the region|

#### GLD ERRORS

| Number | Description |
| :---:| ---      |
| 5100 | Rename the gld to make it appear missing|
| 5101 | GLD env var is set to empty string (same as not existing)|

#### REGION

| Number | Description |
| :---:| ---      |
| 5120 | Rename the default.dat file to make it appear missing|
| 5121 | Check # of sessions by increasing it with a timed session accessing a global|
| 5122 | Verify that processIds of sessions are returned as array|

#### JOURNAL

| Number | Description |
| :---:| ---      |
| 5140 | Switch journaling off in DEFAULT|
| 5141 | Enable journaling in YDBAIM|
| 5142 | Disable journaling in YDBAIM|
| 5143 | Journal file is missing|
| 5144 | Journal file switch: bad region|
| 5145 | Journal file switch: returns OK|

#### REPLICATION

| Number | Description |
| :---:| ---      |
| 5160 | Turn replication on on YDBAIM as verify the response|

#### MAPS

| Number | Description |
| :---:| ---      |
| 5200 | Create a new MAP and see it appearing in the response|

#### LOCKS

| Number | Description |
| :---:| ---      |
| 5220 | LOCKS: Create a lock and verify|
| 5221 | LOCKS: Create a lock and a waiter and verify|
| 5222 | LOCKS: Create a lock and two waiters and verify|
 

#### Get Templates

| Number | Description |
| :---:| ---      |
| 5260 | Verify that BG segment data is complete, including limits|
| 5261 | Verify that MM segment data is complete, including limits|
| 5262 | Verify that region data is complete, including limits|

#### Verify Namespace

| Number | Description |
| :---:| ---      |
| 5270 | Submit just name|
| 5271 | Submit just name with asterisk|
| 5272 | Submit name and subscript|
| 5273 | Submit name and multiple subscripts|
| 5274 | Submit name and multiple subscripts with ranges|
| 5275 | Submit bad name|
| 5276 | Submit name and only open paren|
| 5277 | Submit name and only close paren|
| 5278 | Submit name and bad subscript (alphanumeric, no quotes)|
| 5279 | Submit name and bad subscript (string, only left quote)|
| 5280 | Submit name and bad subscript (string, only right quote)|
| 5281 | Submit name and left range missing|
| 5282 | Submit name and right range missing|
| 5283 | Submit existing name|

#### Verify Filename

| Number | Description |
| :---:| ---      |
| 5290 | Submit name with double /|
| 5291 | Submit name with bad env var|
| 5292 | Submit valid name with existing file|
| 5293 | Submit valid name with absolute path|
| 5294 | Submit valid name with relative path|
| 5295 | Submit valid name with env vars and check for proper extension|
| 5296 | path doesn't exist|
 

#### Create Region

| Number | Description |
| :---:| ---      |
| 5310 | Create random region with default params and verify creation + file existence|
| 5311 | Create random region with different filename and verify creation + file existence|
| 5312 | Create random region with default params, but no db file creation and verify + no file|
| 5313 | Create random region with default params and journal, turned on|
| 5314 | Create random region with default params and journal, turned off|
| 5315 | Create random region with default params and journal, turned off, with different journal path|
| 5316 | Create random region with Auto Db = on|
| 5317 | Create random region with Access Method = MM|
| 5318 | Create random region with all dbAccess fields different and verify them all|
| 5319 | Create random region with all fields different and verify them all|
| 5320 | Create random region with asyncio =true and blocksize= 8192 and verify them all|
| 5321 | Create random region with journal/ON and default fields|
| 5322 | Create random region with journal/OFF and default fields|
| 5323 | Create random region with all journal fields different and verify them all|
| 5324 | Create random region with all segment field different, store them on template, create a new region and verify|
| 5325 | Create random region with all journal field different, store them on template, create a new region and verify|
| 5326 | Add region with fields from Region and Journal changed|

#### Delete Region

| Number | Description |
| :---:| ---      |
| 5400 | Delete using a bad region name|
| 5401 | Delete using a null region name|
| 5402 | Delete an existing region|
| 5403 | Delete an existing region and delete the files|
| 5404 | Delete an existing region and delete the files with segment name different than region name|

#### Journal files

| Number | Description |
| :---:| ---      |
| 6100 | Call the REST /regions/[region]/getJournalInfo with a bad region, error should be returned|
| 6101 | Create a full view situation (multiple journals, with orphans, and active journal file and verify that response is correct|
 

#### Edit Region

| Number | Description |
| :---:| ---      |
| 5340 | Edit YDBOCTO, add one name and delete another, submit and verify|
| 5341 | Edit RANDOM, change BG to MM, submit and verify|
| 5342 | Edit RANDOM, change MM back to BG, submit and verify|
| 5343 | Edit DEFAULT, switch journal OFF, submit and verify|
| 5344 | Edit DEFAULT, switch journal back ON, submit and verify|
| 5345 | Edit DEFAULT, change AutoDB to true and verify|
| 5346 | Edit YDBAIM, change AutoDB to false and verify|
| 5347 | Edit YDBAIM, change AutoDB back to true, change filename, submit and verify|
| 5347 | Edit YDBAIM, change AutoDB back to true, change filename, submit and verify|
| 5348 | Edit DEFAULT, change all Segment related fields, submit and verify|
| 5349 | Edit DEFAULT, change all Segment and Region related fields, submit and verify|
| 5350 | Edit DEFAULT, change all Journaling and Region related fields, submit and verify|
| 5351 | Edit DEFAULT, change all Journaling, Region and Segment related fields, submit and verify|
| 5352 | Edit YDBOCTO, add one name and delete another, change region and journaling and verify|
| 5353 | Edit YDBAIM, add one name and delete another, switch journaling on and verify|
| 5354 | Edit YDBAIM, add one name and delete another, switch journaling off and verify|
| 5355 | Edit DEFAULT, change the journal filename on MUPIP, update and verify|
| 5356 | Edit DEFAULT, change the journal filename on GDE, update and verify\n|
| 5357 | Edit DEFAULT, change the journal filename on GDE, update and verify\n|
 

#### Get All Locks

| Number | Description |
| :---:| ---      |
| 5420 | Set Lock: ^%ydbocto, no waiters, verify|
| 5421 | Set Lock  ^%ydbaim, 1 waiter, verify|
| 5422 | Set Lock  ^%ydbaim, 2 waiters, verify|
| 5423 | Set Lock: test, no waiters, verify|
| 5424 | Set Lock: test, 1 waiter, verify|
| 5425 | Set Lock: test, 1 waiter, verify|
| 5426 | Set Lock  test, test2, ^test3, no waiters, verify|
| 5427 | Set Lock  test, test2, ^test3, 2 waiters, verify|
| 5428 | No locks, verify|
| 5429 | Set lock ^test(\"test with spaces\|

#### Clear lock

| Number | Description |
| :---:| ---      |
| 5430 | No namespace, verify correct error|
| 5431 | Not existing namespace, verify correct error|
| 5432 | Correct namespace, verify correct response and lock gone|
| 5433 | Clear lock in RO mode|

#### Terminate process

| Number | Description |
| :---:| ---      |
| 5435 | Not existing process Id, verify correct error|
| 5436 | process Id 0, verify correct error|
| 5437 | process Id is not YDB, verify correct error|
| 5438 | process Id is correct, verify correct response and terminated process|
 

#### Find globals

| Number | Description |
| :---:| ---      |
| 5500 | List globals using no mask parameter|
| 5501 | List globals using an empty mask|
| 5502 | List globals using AA as mask|
| 5503 | List globals using AAA as mask|
| 5504 | List globals using AAB as mask|
| 5505 | List globals using CC as mask|
| 5506 | List globals using CCCCC|
| 5507 | List globals using CCCC1|
| 5508 | List globals using DDDZ|
| 5509 | List globals using DDDa|

#### getData

| Number | Description |
| :---:| ---      |
| 5525 | Try to supply no path at all|
| 5526 | Try to supply no path at all|
| 5527 | Try to get the path and check its length|

#### dollarData

| Number | Description |
| :---:| ---      |
| 5530 | Verify that a bad name global returns -1|
| 5531 | Verify that a not existing global returns the correct $data value: expect 0|
| 5532 |  Verify that a global returns the correct $data value: expect 1|
| 5533 | Verify that a global returns the correct $data value: expect 10|
| 5534 | Verify that a global returns the correct $data value: expect 11|
 

#### Traverse globals >>> Records validation

| Number | Description |
| :---:| ---      |
| 5551 | Ask for ^PSNDF, no subscripts, 10 entries|
| 5552 | Ask for ^PSNDF, no subscripts, 1000 entries|
| 5553 | Ask for ^PSNDF, no subscripts, 10000 entries|

#### Traverse globals >>> Commas and star after global: regular

| Number | Description |
| :---:| ---      |
| 5570 | ^ORD(,)|
| 5571 | ^ORD(,,)|
| 5572 | ^ORD(,,,)|
| 5573 | ^ORD(*)|
| 5574 | ^ORD(,*)|
| 5575 | ^ORD(,,*)|

#### Traverse globals >>> Commas and star after global: All records

| Number | Description |
| :---:| ---      |
| 5580 | ^ORD(,)|
| 5581 | ^ORD(,,)|
| 5582 | ^ORD(,,,)|
| 5583 | ^ORD(*)|
| 5584 | ^ORD(,*)|
| 5585 | ^ORD(,,*)|

#### Traverse globals >>> Commas and start after 1st subscript: Regular

| Number | Description |
| :---:| ---      |
| 5590 | ^ORD(100.01)|
| 5591 | ^ORD(100.01,)|
| 5592 | ^ORD(100.01,,)|
| 5593 | ^ORD(100.01,,,)|
| 5594 | ^ORD(100.01,*)|
| 5595 | ^ORD(100.01,,*)|
| 5596 | ^ORD(100.01,,,*)|

#### Traverse globals >>> Commas and start after 1st subscript: All records

| Number | Description |
| :---:| ---      |
| 5600 | ^ORD(100.01)|
| 5601 | ^ORD(100.01,)|
| 5602 | ^ORD(100.01,,)|
| 5603 | ^ORD(100.01,,,)|
| 5604 | ^ORD(100.01,*)|
| 5605 | ^ORD(100.01,,*)|
| 5606 | ^ORD(100.01,,,*)|

#### Traverse globals >>>  2 subscripts, numeric and string: Regular

| Number | Description |
| :---:| ---      |
| 5610 | ^ORD(100.01,\"B\")|
| 5611 | ^ORD(100.01,\"B\",)|
| 5612 | ^ORD(100.01,\"B\",,)|
| 5613 | ^ORD(100.01,\"B\",*)|
| 5614 | ^ORD(100.01,\"AMASTERVUID\")|
| 5615 | ^ORD(100.01,\"AMASTERVUID\",)|
| 5616 | ^ORD(100.01,\"AMASTERVUID\",,)|
| 5617 | ^ORD(100.01,\"AMASTERVUID\",*)|

#### Traverse globals >>>  2 subscripts, numeric and string: All records

| Number | Description |
| :---:| ---      |
| 5620 | ^ORD(100.01,\"B\")|
| 5621 | ^ORD(100.01,\"B\",)|
| 5622 | ^ORD(100.01,\"B\",,)|
| 5623 | ^ORD(100.01,\"B\",*)|
| 5624 | ^ORD(100.01,\"AMASTERVUID\")|
| 5625 | ^ORD(100.01,\"AMASTERVUID\",)|
| 5626 | ^ORD(100.01,\"AMASTERVUID\",,)|
| 5627 | ^ORD(100.01,\"AMASTERVUID\",*)|
 

#### Traverse globals >>> Range test

| Number | Description |
| :---:| ---      |
| 5700 | ^ORD(100.01,\"AN\":\"AZ\",*) |
| 5701 | ^ORD(100.01,\"AN\":\"AZ\",4501091:4501096,*) |
| 5702 | ^ORD(100.01,:\"AZ\",*) |
| 5703 | ^ORD(100.01,\"AN\":,*) |
| 5704 | ^ORD(100.01,\"AN\":\"AZ\",4501091:,*) |
| 5705 | ^ORD(100.01,\"AN\":\"AZ\",:4501096,*) |
 

#### Traverse globals >>> With forward offset

| Number | Description |
| :---:| ---      |
| 5750 | ^ORD(100.01,*) 15 recs|

#### Traverse globals >>> With backward offset

| Number | Description |
| :---:| ---      |
| 5800 | ^ORD(100.01,*) 100 recs then the first 10|

#### Traverse globals >>> Test with different dataSize param value

| Number | Description |
| :---:| ---      |
| 5825 | Set dataSize to 10 and verify|
| 5826 | Set dataSize to 5 and verify|
| 5827 | do NOT set dataSize and verify|
| 5828 | Jump to end|
 

#### routines/find

| Number | Description |
| :---:| ---      |
| 5900 | submit with no mask, expect error|
| 5901 | Submit with mask=a, expect error|
| 5902 | Submit with mask=a*, expect error|
| 5903 | Submit with mask=%*, expect error|
| 5904 | Submit with mask=%?, expect error|
| 5905 | Submit with mask=a?, expect error|
| 5906 | Submit with mask=%ydb*, expect ok|
| 5907 | Submit with mask=%ydb*, verify payload|

#### routines/{routine}}

| Number | Description |
| :---:| ---      |
| 5925 | submit no routine, expect error|
| 5926 | submit bad routine, expect error|
| 5927 | submit good routine, expect ok|
| 5928 | submit good routine with no object, verify payload|
| 5929 | submit good routine, verify payload|
 

#### Maintenance: REORG

| Number | Description |
| :---:| ---      |
| 6000 | call /api/regions/maintenance/defrag with no body|
| 6001 | call /api/regions/maintenance/defrag with no fillFactor|
| 6002 | call /api/regions/maintenance/defrag with no indexFillFactor|
| 6003 | call /api/regions/maintenance/defrag with no regions|
| 6005 | call /api/regions/maintenance/defrag with valid body and YDBAIM and YDBOCTO, should return no globals found on both|
| 6006 | create globals on DEFAULT, call /api/regions/maintenance/defrag with valid body and DEFAULT and YDBOCTO, should return no globals found on YDBOCTO and valid data on 'DEFAULT'|
| 6007 | call /api/regions/maintenance/defrag with valid body and DEFAULT, verify that globals list is returned as array in response payloadRegions|
| 6008 | call /api/regions/maintenance/defrag in RO mode, expect HTTP 403|

#### Maintenance: INTEG

| Number | Description |
| :---:| ---      |
| 6025 | call /api/regions/maintenance/integ with no body|
| 6026 | call /api/regions/maintenance/integ with no regions|
| 6027 | call /api/regions/maintenance/integ with no bad type|
| 6028 | call /api/regions/maintenance/integ with no bad reporting|
| 6029 | call /api/regions/maintenance/integ with keyRange different than boolean|
| 6030 | call /api/regions/maintenance/integ with stat different than boolean|
| 6031 | call /api/regions/maintenance/integ with mapErrors with negative number|
| 6032 | call /api/regions/maintenance/integ with mapErrors with number > 1,000,000|
| 6033 | call /api/regions/maintenance/integ with keySizeErrors with negative number|
| 6034 | call /api/regions/maintenance/integ with keySizeErrors with number > 1,000,000|
| 6035 | call /api/regions/maintenance/integ with transactionErrors with negative number|
| 6036 | call /api/regions/maintenance/integ with transactionErrors with number > 1,000,000|
| 6037 | call /api/regions/maintenance/integ with reporting = full, verify|
| 6038 | call /api/regions/maintenance/integ with reporting = summary, verify|
| 6039 | call /api/regions/maintenance/integ in RO mode, expect HTTP 403|

#### Maintenance: BACKUP

| Number | Description |
| :---:| ---      |
| 6050 | call /api/regions/maintenance/backup with no body|
| 6051 | call /api/regions/maintenance/backup with no regions|
| 6052 | call /api/regions/maintenance/backup with no targetPath|
| 6053 | call /api/regions/maintenance/backup with no replace|
| 6054 | call /api/regions/maintenance/backup with no disableJournaling|
| 6055 | call /api/regions/maintenance/backup with no record|
| 6056 | call /api/regions/maintenance/backup with no createNewJournalFiles|
| 6057 | call /api/regions/maintenance/backup with createNewJournalFiles = 'link'|
| 6058 | call /api/regions/maintenance/backup with no includeReplicatedInstances|
| 6059 | call /api/regions/maintenance/backup with includeReplicatedInstances = true|
| 6060 | call /api/regions/maintenance/backup with includeReplicatedInstances = true, replUseSamePath = false|
| 6061 | call /api/regions/maintenance/backup in RO mode, expect HTTP 403|
| 6062 | perform a backup and confirm response|
 
