/****************************************************************
 *                                                              *
 * Copyright (c) 2022-2023 YottaDB LLC and/or its subsidiaries.      *
 * All rights reserved.                                         *
 *                                                              *
 * This source code contains the intellectual property          *
 * of its copyright holder(s), and is made available            *
 * under a license.  If you do not know the terms of            *
 * the license, please stop and do not read further.            *
 *                                                              *
 ****************************************************************/

const libs = require('../../../libs');
const {expect} = require("chai");
const {execSync, exec} = require('child_process');

describe("CLIENT: Globals > Table: general", async () => {
    execSync('wwwroot/test/globals/createglobals.sh');

    it("Test # 1200: Submit a query to ^ORD and verify that the record size is correct", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1200`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.waitForDialog('#modalDashboard');

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(100);

        const $tbody = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody').find('tr'));
        expect($tbody.length >= 200).to.be.true
    });

    it("Test # 1201: Data view: regular: Submit a query to ^ORD and verify that data is correct", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1201`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.waitForDialog('#modalDashboard');

        // switch to data view: normal
        await page.evaluate(() => app.ui.gViewer.table.changeRightView('-G-1', 'no'));

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        const $data = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody > tr:first-child td:nth-child(3)').text());
        expect($data).to.have.string('"ORDER STATUS^100.01I^99^16"')
    });

    it("Test # 1202: Data view: pieces: Submit a query to ^ORD and verify that data is correct", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1202`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.waitForDialog('#modalDashboard');

        // switch to data view: pieces
        await page.evaluate(() => app.ui.gViewer.table.changeRightView('-G-1', 'yes'));

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        const $data = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody > tr:first-child td:nth-child(3)').text());
        expect($data).to.have.string('"ORDER STATUS^100.01I^99^16"')
    });

    it("Test # 1204: Data view: pieces: Change piece char, submit a query to ^ORD and verify that data is correct", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1204`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.waitForDialog('#modalDashboard');

        // switch to data view: pieces
        await page.evaluate(() => app.ui.gViewer.table.changePieceChar('-G-1', '.'));
        await page.evaluate(() => app.ui.gViewer.table.changeRightView('-G-1', 'yes'));

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        let $data = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody > tr:first-child td:nth-child(3)').text());
        expect($data).to.have.string('ORDER STATUS^100.01I^99^16')
    });

    it("Test # 1205: Data view: pills: Change piece char, submit a query to ^ORD and verify that data is correct", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1205`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.waitForDialog('#modalDashboard');

        // switch to data view: pieces
        await page.evaluate(() => app.ui.gViewer.table.changePieceChar('-G-1', '.'));
        await page.evaluate(() => app.ui.gViewer.table.changeRightView('-G-1', 'pills'));

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        let $data = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody > tr:first-child td:nth-child(3)').text());
        expect($data).to.have.string('P1 ORDER STATUS^100 P2 01I')
    });

    it("Test # 1206: Data view: force the data size to 16 chars verify that data length gets trimmed", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1206`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.delay(2500)

        // switch to data view: pieces
        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.maxDataLength = 16);
        await page.evaluate(() => app.ui.gViewer.table.changeRightView('-G-1', 'no'));

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        let $data = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody > tr:first-child td:nth-child(3)').text());
        expect($data).to.have.string('"ORDER STATUS^100"…')
    });

    it("Test # 1207: Data view: force the data size to 16 chars verify that a button is present", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1207`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.waitForDialog('#modalDashboard');

        // switch to 16 chars max length
        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.maxDataLength = 16);
        await page.evaluate(() => app.ui.gViewer.table.changeRightView('-G-1', 'no'));

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        let $data = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody > tr:first-child td:nth-child(3)').find('button'));
        expect($data.length === 1).to.be.true;
    });

    it("Test # 1208: switch to dark mode and verify that table changes appearance", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1208`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.waitForDialog('#modalDashboard');

        // switch to dark mode
        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.appearance.theme = 'dark');

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        let $data = await page.evaluate(() => $('.table-dark'));
        expect($data.length === 1).to.be.true;
    });

    it("Test # 1209: Zoom in and verify that font is bigger", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1209`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.waitForDialog('#modalDashboard');

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        const oldFontSize = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody > tr:first-child td:nth-child(1)').css('font-size'));

        // zoom in
        await page.evaluate(() => app.ui.gViewer.table.zoom('-G-1', '+'));

        await libs.delay(250);

        const newFontSize = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody > tr:first-child td:nth-child(1)').css('font-size'));

        expect(newFontSize > oldFontSize).to.be.true;
    });

    it("Test # 1210: Zoom out and verify that font is smaller", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1210`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.waitForDialog('#modalDashboard');

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        const oldFontSize = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody > tr:first-child td:nth-child(1)').css('font-size'));

        // zoom out
        await page.evaluate(() => app.ui.gViewer.table.zoom('-G-1', '-'));

        await libs.delay(250);

        const newFontSize = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody > tr:first-child td:nth-child(1)').css('font-size'));

        expect(newFontSize < oldFontSize).to.be.true;
    });
    // COMMENTED OUT BECAUSE OF FAILURES IN SERVER, WILL BE FIXED ON NEXT SPRINT

    /*
    it("Test # 1211: Verify that the default bookmark color is loaded", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1211`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.waitForDialog('#modalDashboard');

        let defaultBgnd = await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.table.bookmarkColor);
        if (defaultBgnd === 'red') defaultBgnd = 'rgb(255, 0, 0)';

        const background = await page.evaluate(() => $('#lblGviewerBookmarkColor-G-1').css('background-color'));
        console.log(background)

        expect(background === defaultBgnd).to.be.true;
    });

    it("Test # 1212: Verify that the default data view is loaded", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1212`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.waitForDialog('#modalDashboard');

        let defaultDataMode = await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.table.rows.right.showPieces);
        if (defaultDataMode === 'pills') defaultDataMode = true;

        const hasPills = await page.evaluate(() => $('#lblGviewerToolbarPiecesPills-G-1').hasClass('active'));

        expect(defaultDataMode === hasPills).to.be.true;
    });

    it("Test # 1213: Verify that the default piece char is loaded", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1213`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.delay(250);

        let defaultPieceChar = await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.table.rows.right.pieceChar);

        const caption = await page.evaluate(() => $('#btnGviewerPieceChar-G-1').text());

        expect(caption).to.have.string(defaultPieceChar)
    });

    it("Test # 1214: Verify that the default theme is loaded", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1214`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.delay(250);

        let defaultTheme = await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.appearance.theme);

        let theme = await page.evaluate(() => $('#btnGviewerPieceChar-G-1').hasClass('active'));
        if (theme === false) theme = 'light';

        expect(defaultTheme).to.have.string(theme)
    });
     */

});

describe("CLIENT: Globals > Table: fetches", async () => {
    it("Test # 1250: Set the max buffer to 100, initial fetch to 50: verify that data size = 50", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1250`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.delay(250);

        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.maxSize = 250);
        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.initialFetchSize = 50);
        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.lazyFetchSize = 25);

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(100);

        const $tbody = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody').find('tr'));
        expect($tbody.length === 50).to.be.true
    });

    it("Test # 1251: Set the max buffer to 100, initial fetch to 50 and lazy to 25: verify that, scrolling down, it is fetching", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1251`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.delay(250);

        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.maxSize = 100);
        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.initialFetchSize = 50);
        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.lazyFetchSize = 25);

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        await page.evaluate(() => $('#divGviewerTableScroll-G-1').scrollTop(10000) );

        await libs.delay(500);

        const $tbody = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody').find('tr'));
        expect($tbody.length === 75).to.be.true
    });

    it("Test # 1252: Set the max buffer to 100, initial fetch to 50 and lazy to 25: verify that, scrolling down after buffer is full , it is fetching and beheading the buffer", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1252`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.delay(250);

        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.maxSize = 100);
        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.initialFetchSize = 50);
        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.lazyFetchSize = 25);

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        await page.evaluate(() => $('#divGviewerTableScroll-G-1').scrollTop(10000) );
        await libs.delay(250);

        await page.evaluate(() => $('#divGviewerTableScroll-G-1').scrollTop(30000) );
        await libs.delay(250);

        await page.evaluate(() => $('#divGviewerTableScroll-G-1').scrollTop(30000) );
        await libs.delay(250);

        const firstRowId = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody > tr:first-child').attr('id'));
        expect(firstRowId === 'rowTableGviewer-G-1-25').to.be.true
    });

    it("Test # 1253: Set the max buffer to 100, initial fetch to 50 and lazy to 25: verify that, scrolling down after buffer is full and up again, it is fetching and betailing the buffer", async () => {
        await page.goto(`https://localhost:${MDevPort}//index.html?test=1253`, {
            waitUntil: "domcontentloaded"
        });

        // wait for dashboard to be set by the async call
        await libs.delay(250);

        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.maxSize = 100);
        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.initialFetchSize = 50);
        await page.evaluate(() => app.ui.gViewer.instance['-G-1'].viewer.buffer.defaults.buffering.lazyFetchSize = 25);

        //type a new name
        await page.keyboard.type('ORD');
        await page.keyboard.press('Enter');

        await libs.delay(250);

        await page.evaluate(() => $('#divGviewerTableScroll-G-1').scrollTop(10000) );
        await libs.delay(250);

        await page.evaluate(() => $('#divGviewerTableScroll-G-1').scrollTop(30000) );
        await libs.delay(250);

        await page.evaluate(() => $('#divGviewerTableScroll-G-1').scrollTop(30000) );
        await libs.delay(250);

        await page.evaluate(() => $('#divGviewerTableScroll-G-1').scrollTop(-200000));
        await libs.delay(500);

        //await page.keyboard.down('Control');
        await page.keyboard.press('Home');

        await libs.delay(500);

        const $tbody = await page.evaluate(() => $('#tblGviewerTable-G-1 > tbody').find('tr'));
        expect($tbody.length === 100).to.be.true
    });

});
